@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: internal.c,v 1.67.2.3 2014/07/04 07:45:43 markisch Exp $"); }
+static char *RCSid() { return RCSid("$Id: internal.c,v 1.96.2.3 2017/08/30 18:37:07 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - internal.c */
@@ -39,13 +39,21 @@ static char *RCSid() { return RCSid("$Id: internal.c,v 1.67.2.3 2014/07/04 07:45
 
 #include "stdfn.h"
 #include "alloc.h"
-#include "util.h"		/* for int_error() */
-#include "gp_time.h"           /* for str(p|f)time */
-#include "command.h"            /* for do_system_func */
-#include "variable.h" /* For locale handling */
+#include "util.h"	/* for int_error() */
+#include "gp_time.h"	/* for str(p|f)time */
+#include "command.h"	/* for do_system_func */
+#include "datablock.h"	/* for datablock_size() */
+#include "variable.h"	/* for locale handling */
+#include "parse.h"	/* for string_result_only */
 
 #include <math.h>
 
+#if !defined(__MINGW64_VERSION_MAJOR)
+/*
+ * FIXME: This is almost certainly out of date on linux, since the matherr
+ * mechanism has been replaced by math_error() and supposedly is only 
+ * enabled via an explicit declaration #define _SVID_SOURCE.
+ */
 /*
  * Excerpt from the Solaris man page for matherr():
  *
@@ -56,9 +64,6 @@ static char *RCSid() { return RCSid("$Id: internal.c,v 1.67.2.3 2014/07/04 07:45
  *   matherr() in their programs.
  */
 
-static enum DATA_TYPES sprintf_specifier __PROTO((const char *format));
-
-#ifndef _WIN64
 int
 GP_MATHERR( STRUCT_EXCEPTION_P_X )
 {
@@ -66,7 +71,12 @@ GP_MATHERR( STRUCT_EXCEPTION_P_X )
 }
 #endif
 
-#define BAD_DEFAULT default: int_error(NO_CARET, "internal error : type neither INT or CMPLX"); return;
+static enum DATA_TYPES sprintf_specifier __PROTO((const char *format));
+
+#define BADINT_DEFAULT int_error(NO_CARET, "error: bit shift applied to non-INT");
+#define BAD_TYPE(type) int_error(NO_CARET, (type==NOTDEFINED) ? "uninitialized user variable" : "internal error : type neither INT nor CMPLX");
+	
+static const char *nonstring_error = "internal error : STRING operator applied to undefined or non-STRING variable";
 
 static int recursion_depth = 0;
 void
@@ -81,8 +91,12 @@ f_push(union argument *x)
     struct udvt_entry *udv;
 
     udv = x->udv_arg;
-    if (udv->udv_undef) {	/* undefined */
-	int_error(NO_CARET, "undefined variable: %s", udv->udv_name);
+    if (udv->udv_value.type == NOTDEFINED) {
+	if (string_result_only)
+	/* We're only here to check whether this is a string. It isn't. */
+	    udv = udv_NaN;
+	else
+	    int_error(NO_CARET, "undefined variable: %s", udv->udv_name);
     }
     push(&(udv->udv_value));
 }
@@ -105,7 +119,12 @@ f_pop(union argument *x)
 {
     struct value dummy;
     pop(&dummy);
-    gpfree_string(&dummy);
+    if (dummy.type == STRING)
+	gpfree_string(&dummy);
+#ifdef ARRAY_COPY_ON_REFERENCE
+    if (dummy.type == ARRAY)
+	gpfree_string(&dummy);
+#endif
 }
 
 void
@@ -132,12 +151,22 @@ f_call(union argument *x)
     struct value save_dummy;
 
     udf = x->udf_arg;
-    if (!udf->at)
+    if (!udf->at) {
+	if (string_result_only) {
+	    /* We're only here to check whether this is a string. It isn't. */
+	    f_pop(x);
+	    push(&(udv_NaN->udv_value));
+	    return;
+	}
 	int_error(NO_CARET, "undefined function: %s", udf->udf_name);
+    }
 
     save_dummy = udf->dummy_values[0];
     (void) pop(&(udf->dummy_values[0]));
 
+    if (udf->dummy_values[0].type == ARRAY)
+	int_error(NO_CARET, "f_call: unsupported array operation");
+
     if (udf->dummy_num != 1)
 	int_error(NO_CARET, "function %s requires %d variables", udf->udf_name, udf->dummy_num);
 
@@ -172,7 +201,7 @@ f_calln(union argument *x)
     (void) pop(&num_params);
 
     if (num_params.v.int_val != udf->dummy_num)
-	int_error(NO_CARET, "function %s requires %d variable%c",
+	int_error(NO_CARET, "function %s requires %d variable%c", 
 	    udf->udf_name, udf->dummy_num, (udf->dummy_num == 1)?'\0':'s');
 
     /* if there are more parameters than the function is expecting */
@@ -189,8 +218,11 @@ f_calln(union argument *x)
     }
 
     /* pop parameters we can use */
-    for (i = num_pop - 1; i >= 0; i--)
+    for (i = num_pop - 1; i >= 0; i--) {
 	(void) pop(&(udf->dummy_values[i]));
+	if (udf->dummy_values[i].type == ARRAY)
+	    int_error(NO_CARET, "f_calln: unsupported array operation");
+    }
 
     if (recursion_depth++ > STACK_DEPTH)
 	int_error(NO_CARET, "recursion depth limit exceeded");
@@ -228,7 +260,6 @@ f_sum(union argument *arg)
     udv = get_udv_by_name(varname.v.string_val);
     if (!udv)
         int_error(NO_CARET, "internal error: f_sum could not access iteration variable.");
-    udv->udv_undef = false;
 
     udf = arg->udf_arg;
     if (!udf)
@@ -358,7 +389,9 @@ f_uminus(union argument *arg)
 	a.v.cmplx_val.imag =
 	    -a.v.cmplx_val.imag;
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
+	break;
     }
     push(&a);
 }
@@ -387,7 +420,8 @@ f_eq(union argument *arg)
 		      b.v.cmplx_val.real &&
 		      b.v.cmplx_val.imag == 0.0);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -402,10 +436,12 @@ f_eq(union argument *arg)
 		      a.v.cmplx_val.imag ==
 		      b.v.cmplx_val.imag);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(Ginteger(&a, result));
 }
@@ -432,7 +468,8 @@ f_ne(union argument *arg)
 		      b.v.cmplx_val.real ||
 		      b.v.cmplx_val.imag != 0.0);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -448,10 +485,12 @@ f_ne(union argument *arg)
 		      a.v.cmplx_val.imag !=
 		      b.v.cmplx_val.imag);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(Ginteger(&a, result));
 }
@@ -477,7 +516,8 @@ f_gt(union argument *arg)
 	    result = (a.v.int_val >
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -490,10 +530,12 @@ f_gt(union argument *arg)
 	    result = (a.v.cmplx_val.real >
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(Ginteger(&a, result));
 }
@@ -519,7 +561,8 @@ f_lt(union argument *arg)
 	    result = (a.v.int_val <
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -532,10 +575,12 @@ f_lt(union argument *arg)
 	    result = (a.v.cmplx_val.real <
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(Ginteger(&a, result));
 }
@@ -561,7 +606,8 @@ f_ge(union argument *arg)
 	    result = (a.v.int_val >=
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -574,10 +620,12 @@ f_ge(union argument *arg)
 	    result = (a.v.cmplx_val.real >=
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(Ginteger(&a, result));
 }
@@ -603,7 +651,8 @@ f_le(union argument *arg)
 	    result = (a.v.int_val <=
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -616,15 +665,69 @@ f_le(union argument *arg)
 	    result = (a.v.cmplx_val.real <=
 		      b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(Ginteger(&a, result));
 }
 
 
+void
+f_leftshift(union argument *arg)
+{
+    struct value a, b, result;
+
+    (void) arg;			/* avoid -Wunused warning */
+    (void) pop(&b);
+    (void) pop(&a);
+    switch (a.type) {
+    case INTGR:
+	switch (b.type) {
+	case INTGR:
+	    (void) Ginteger(&result, (unsigned)(a.v.int_val) << b.v.int_val);
+	    break;
+	default:
+	    BADINT_DEFAULT
+	}
+	break;
+    default:
+	BADINT_DEFAULT
+    }
+    push(&result);
+}
+
+
+
+void
+f_rightshift(union argument *arg)
+{
+    struct value a, b, result;
+
+    (void) arg;			/* avoid -Wunused warning */
+    (void) pop(&b);
+    (void) pop(&a);
+    switch (a.type) {
+    case INTGR:
+	switch (b.type) {
+	case INTGR:
+	    (void) Ginteger(&result, (unsigned)(a.v.int_val) >> b.v.int_val);
+	    break;
+	default:
+	    BADINT_DEFAULT
+	}
+	break;
+    default:
+	BADINT_DEFAULT
+    }
+    push(&result);
+}
+
+
+
 void
 f_plus(union argument *arg)
 {
@@ -645,7 +748,8 @@ f_plus(union argument *arg)
 			    b.v.cmplx_val.real,
 			    b.v.cmplx_val.imag);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -661,10 +765,12 @@ f_plus(union argument *arg)
 			    a.v.cmplx_val.imag +
 			    b.v.cmplx_val.imag);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(&result);
 }
@@ -690,7 +796,8 @@ f_minus(union argument *arg)
 			    b.v.cmplx_val.real,
 			    -b.v.cmplx_val.imag);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -706,10 +813,12 @@ f_minus(union argument *arg)
 			    a.v.cmplx_val.imag -
 			    b.v.cmplx_val.imag);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(&result);
 }
@@ -741,7 +850,8 @@ f_mult(union argument *arg)
 			    a.v.int_val *
 			    b.v.cmplx_val.imag);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -762,10 +872,12 @@ f_mult(union argument *arg)
 			    a.v.cmplx_val.imag *
 			    b.v.cmplx_val.real);
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(&result);
 }
@@ -808,7 +920,8 @@ f_div(union argument *arg)
 		undefined = TRUE;
 	    }
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -844,10 +957,12 @@ f_div(union argument *arg)
 		undefined = TRUE;
 	    }
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
     push(&result);
 }
@@ -929,7 +1044,8 @@ f_power(union argument *arg)
 				mag * sin(ang));
 	    }
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
     case CMPLX:
@@ -980,11 +1096,27 @@ f_power(union argument *arg)
 				mag * sin(ang));
 	    }
 	    break;
-	    BAD_DEFAULT
+	default:
+	    BAD_TYPE(b.type)
 	}
 	break;
-	BAD_DEFAULT
+    default:
+	BAD_TYPE(a.type)
     }
+
+    /* Catch underflow and return 0 */
+    /* Note: fpclassify() is an ISOC99 macro found also in other libc implementations */
+#ifdef fpclassify
+    if (errno == ERANGE && result.type == CMPLX) {
+	int fperror = fpclassify(result.v.cmplx_val.real);
+	if (fperror == FP_ZERO || fperror == FP_SUBNORMAL) {
+	    result.v.cmplx_val.real = 0.0;
+	    result.v.cmplx_val.imag = 0.0;
+	    errno = 0;
+	}
+    }
+#endif
+
     push(&result);
 }
 
@@ -1040,7 +1172,7 @@ f_concatenate(union argument *arg)
     }
 
     if (a.type != STRING || b.type != STRING)
-	int_error(NO_CARET, "internal error : STRING operator applied to non-STRING type");
+	int_error(NO_CARET, nonstring_error);
 
     (void) Gstring(&result, gp_stradd(a.v.string_val, b.v.string_val));
     gpfree_string(&a);
@@ -1058,8 +1190,8 @@ f_eqs(union argument *arg)
     (void) pop(&b);
     (void) pop(&a);
 
-    if(a.type != STRING || b.type != STRING)
-	int_error(NO_CARET, "internal error : STRING operator applied to non-STRING type");
+    if (a.type != STRING || b.type != STRING)
+	int_error(NO_CARET, nonstring_error);
 
     (void) Ginteger(&result, !strcmp(a.v.string_val, b.v.string_val));
     gpfree_string(&a);
@@ -1076,8 +1208,8 @@ f_nes(union argument *arg)
     (void) pop(&b);
     (void) pop(&a);
 
-    if(a.type != STRING || b.type != STRING)
-	int_error(NO_CARET, "internal error : STRING operator applied to non-STRING type");
+    if (a.type != STRING || b.type != STRING)
+	int_error(NO_CARET, nonstring_error);
 
     (void) Ginteger(&result, (int)(strcmp(a.v.string_val, b.v.string_val)!=0));
     gpfree_string(&a);
@@ -1131,42 +1263,124 @@ f_range(union argument *arg)
 {
     struct value beg, end, full;
     struct value substr;
+    int ibeg, iend;
 
     (void) arg;			/* avoid -Wunused warning */
     (void) pop(&end);
     (void) pop(&beg);
     (void) pop(&full);
 
-    if (end.type != INTGR || beg.type != INTGR)
-	int_error(NO_CARET, "internal error: substring range specifiers must have integer values");
-
+    if (beg.type == INTGR)
+	ibeg = beg.v.int_val;
+    else if (beg.type == CMPLX)
+	ibeg = floor(beg.v.cmplx_val.real);
+    else
+	int_error(NO_CARET, "internal error: non-numeric substring range specifier");
+    if (end.type == INTGR)
+	iend = end.v.int_val;
+    else if (end.type == CMPLX)
+	iend = floor(end.v.cmplx_val.real);
+    else
+	int_error(NO_CARET, "internal error: non-numeric substring range specifier");
+	
     if (full.type != STRING)
 	int_error(NO_CARET, "internal error: substring range operator applied to non-STRING type");
 
     FPRINTF((stderr,"f_range( \"%s\", %d, %d)\n", full.v.string_val, beg.v.int_val, end.v.int_val));
 
-    if (end.v.int_val > gp_strlen(full.v.string_val))
-	end.v.int_val = gp_strlen(full.v.string_val);
-    if (beg.v.int_val < 1)
-	beg.v.int_val = 1;
+    if (iend > gp_strlen(full.v.string_val))
+	iend = gp_strlen(full.v.string_val);
+    if (ibeg < 1)
+	ibeg = 1;
 
-    if (beg.v.int_val > end.v.int_val) {
+    if (ibeg > iend) {
 	push(Gstring(&substr, ""));
     } else {
-	char *begp = gp_strchrn(full.v.string_val,beg.v.int_val-1);
-	char *endp = gp_strchrn(full.v.string_val,end.v.int_val);
+	char *begp = gp_strchrn(full.v.string_val,ibeg-1);
+	char *endp = gp_strchrn(full.v.string_val,iend);
 	*endp = '\0';
 	push(Gstring(&substr, begp));
     }
     gpfree_string(&full);
 }
 
+
+/*
+ * f_index() extracts the value of a single element from an array.
+ */
+void
+f_index(union argument *arg)
+{
+    struct value array, index;
+    int i = -1;
+
+    (void) arg;			/* avoid -Wunused warning */
+    (void) pop(&index);
+    (void) pop(&array);
+
+    if (index.type == INTGR)
+	i = index.v.int_val;
+    else if (index.type == CMPLX)
+	i = floor(index.v.cmplx_val.real);
+
+    if (array.type == ARRAY) {
+	if (i <= 0 || i > array.v.value_array[0].v.int_val) 
+	    int_error(NO_CARET, "array index out of range");
+	push( &array.v.value_array[i] );
+
+    } else if (array.type == DATABLOCK) {
+	i--;	/* line numbers run from 1 to nlines */
+	if (i < 0 || i >= datablock_size(&array))
+	    int_error(NO_CARET, "datablock index out of range");
+	push( Gstring(&array, array.v.data_array[i]) );
+    }
+
+    else
+	int_error(NO_CARET, "internal error: attempt to index a scalar variable");
+}
+
+/*
+ * f_cardinality() extracts the number of elements in an array.
+ */
+void
+f_cardinality(union argument *arg)
+{
+    struct value array;
+    int size;
+    (void) arg;			/* avoid -Wunused warning */
+    (void) pop(&array);
+
+    if (array.type == ARRAY)
+	size = array.v.value_array[0].v.int_val;
+    else if (array.type == DATABLOCK)
+	size = datablock_size(&array);
+    else
+	int_error(NO_CARET, "internal error: cardinality of a scalar variable");
+
+    push(Ginteger(&array, size));
+}
+
+/* Magic number! */
+#define RETURN_WORD_COUNT (-17*23*61)
+
 void
 f_words(union argument *arg)
+{
+    struct value a;
+
+    /* "words(s)" is implemented as "word(s,RETURN_WORD_COUNT)" */
+    push(Ginteger(&a, RETURN_WORD_COUNT));
+    f_word(arg);
+}
+
+void
+f_word(union argument *arg)
 {
     struct value a, b, result;
     int nwords = 0;
+    int in_string = 0;
     int ntarget;
+    char q = '\0';
     char *s;
 
     (void) arg;
@@ -1180,28 +1394,39 @@ f_words(union argument *arg)
 
     Gstring(&result, "");
     while (*s) {
-	while (isspace(*s)) s++;
+	while (isspace((unsigned char)*s)) s++;
 	if (!*s)
 	    break;
 	nwords++;
+	if (*s == '"' || *s == '\'') {
+	    q = *s;
+	    s++;
+	    in_string = 1;
+	}
 	if (nwords == ntarget) { /* Found the one we wanted */
 	    Gstring(&result,s);
 	    s = result.v.string_val;
 	}
-	while (*s && !isspace(*s)) s++;
+	while (*s && ((!isspace((unsigned char)*s) && !in_string) || (in_string && *s != q))) s++;
 	if (nwords == ntarget) { /* Terminate this word cleanly */
 	    *s = '\0';
 	    break;
 	}
+	if (in_string && (*s == q)) {
+	    in_string = 0;
+	    s++;
+	}
     }
 
-    if (ntarget < 0)
-	/* words(s) = word(s,-1) = # of words in string */
+    /* words(s) = word(s,magic_number) = # of words in string */
+    if (ntarget == RETURN_WORD_COUNT)
 	Ginteger(&result, nwords);
 
     push(&result);
     gpfree_string(&a);
 }
+#undef RETURN_WORD_COUNT
+
 
 /* EAM July 2004  (revised to dynamic buffer July 2005)
  * There are probably an infinite number of things that can
@@ -1223,10 +1448,10 @@ f_sprintf(union argument *arg)
     int prev_pos;
     int i, remaining;
     int nargs = 0;
-    int save_errno;
     enum DATA_TYPES spec_type;
 
     /* Retrieve number of parameters from top of stack */
+    (void) arg;
     pop(&num_params);
     nargs = num_params.v.int_val;
     if (nargs > 10) {	/* Fall back to slow but sure allocation */
@@ -1292,7 +1517,6 @@ f_sprintf(union argument *arg)
 
 #ifdef HAVE_SNPRINTF
 	/* Use the format to print next arg */
-	save_errno = errno;
 	switch(spec_type) {
 	case INTGR:
 	    snprintf(outpos,bufsize-(outpos-buffer),
@@ -1309,11 +1533,6 @@ f_sprintf(union argument *arg)
 	default:
 	    int_error(NO_CARET,"internal error: invalid spec_type");
 	}
-#if _MSC_VER
-       buffer[bufsize-1] = '\0';	/* VC++ is not ANSI-compliant */
-       if (errno == ERANGE) errno = save_errno;
-#endif
-
 #else
 	/* FIXME - this is bad; we should dummy up an snprintf equivalent */
 	switch(spec_type) {
@@ -1389,19 +1608,12 @@ f_gprintf(union argument *arg)
     char *buffer;
     int length;
     double base = 10.;
-
+ 
     /* Retrieve parameters from top of stack */
+    (void) arg;
     pop(&val);
     pop(&fmt);
 
-#ifdef DEBUG
-    fprintf(stderr,"----------\nGot gprintf parameters\nfmt: ");
-	disp_value(stderr, &fmt, TRUE);
-    fprintf(stderr,"\nval: ");
-	disp_value(stderr, &val, TRUE);
-    fprintf(stderr,"\n----------\n");
-#endif
-
     /* Make sure parameters are of the correct type */
     if (fmt.type != STRING)
 	int_error(NO_CARET,"First parameter to gprintf must be a format string");
@@ -1503,13 +1715,13 @@ f_strptime(union argument *arg)
     push(Gcomplex(&val, result, 0.0));
 }
 
-/* Get current system time in seconds since 2000
- * The type of the value popped from the stack
+/* Get current system time in seconds since 2000 
+ * The type of the value popped from the stack 
  * determines what is returned.
  * If integer, the result is also an integer.
- * If real (complex), the result is also real,
+ * If real (complex), the result is also real, 
  * with microsecond precision (if available).
- * If string, it is assumed to be a format string,
+ * If string, it is assumed to be a format string, 
  * and it is passed to strftime to get a formatted time string.
  */
 void
@@ -1530,8 +1742,8 @@ f_time(union argument *arg)
 #endif
 
     (void) arg; /* Avoid compiler warnings */
-    pop(&val);
-
+    pop(&val); 
+    
     switch(val.type) {
 	case INTGR:
 	    push(Ginteger(&val, (int) time_now));
@@ -1601,11 +1813,11 @@ void
 f_system(union argument *arg)
 {
     struct value val, result;
-    struct udvt_entry *errno_var;
     char *output;
     int output_len, ierr;
 
     /* Retrieve parameters from top of stack */
+    (void) arg;
     pop(&val);
 
     /* Make sure parameters are of the correct type */
@@ -1615,13 +1827,10 @@ f_system(union argument *arg)
     FPRINTF((stderr," f_system input = \"%s\"\n", val.v.string_val));
 
     ierr = do_system_func(val.v.string_val, &output);
-    if ((errno_var = add_udv_by_name("ERRNO"))) {
-	errno_var->udv_undef = FALSE;
-	Ginteger(&errno_var->udv_value, ierr);
-    }
-    output_len = strlen(output);
+    fill_gpval_integer("GPVAL_ERRNO", ierr); 
 
     /* chomp result */
+    output_len = strlen(output);
     if ( output_len > 0 && output[output_len-1] == '\n' )
 	output[output_len-1] = NUL;
 
@@ -1638,25 +1847,41 @@ f_system(union argument *arg)
 void
 f_assign(union argument *arg)
 {
-    struct value a, b;
+    struct udvt_entry *udv;
+    struct value a, b, index;
     (void) arg;
     (void) pop(&b);	/* new value */
+    (void) pop(&index);	/* index (only used if this is an array assignment) */
     (void) pop(&a);	/* name of variable */
-
-    if (a.type == STRING) {
-	struct udvt_entry *udv;
-	if (!strncmp(a.v.string_val,"GPVAL_",6) || !strncmp(a.v.string_val,"MOUSE_",6))
-	    int_error(NO_CARET,"Attempt to assign to a read-only variable");
-	udv = add_udv_by_name(a.v.string_val);
-	gpfree_string(&a);
-	if (!udv->udv_undef)
-	    gpfree_string(&(udv->udv_value));
-	udv->udv_value = b;
-	udv->udv_undef = FALSE;
-	push(&b);
-    } else {
+    
+    if (a.type != STRING)
 	int_error(NO_CARET, "attempt to assign to something other than a named variable");
+    if (!strncmp(a.v.string_val,"GPVAL_",6) || !strncmp(a.v.string_val,"MOUSE_",6))
+	int_error(NO_CARET, "attempt to assign to a read-only variable");
+    if (b.type == ARRAY)
+	int_error(NO_CARET, "unsupported array operation");
+
+    udv = add_udv_by_name(a.v.string_val);
+    gpfree_string(&a);
+
+    if (udv->udv_value.type == ARRAY) {
+	int i;
+	if (index.type == INTGR)
+	    i = index.v.int_val;
+	else if (index.type == CMPLX)
+	    i = floor(index.v.cmplx_val.real);
+	else
+	    int_error(NO_CARET, "non-numeric array index");
+	if (i <= 0 || i > udv->udv_value.v.value_array[0].v.int_val)
+	    int_error(NO_CARET, "array index out of range");
+	gpfree_string(&udv->udv_value.v.value_array[i]);
+	udv->udv_value.v.value_array[i] = b;
+    } else {
+	gpfree_string(&(udv->udv_value));
+	udv->udv_value = b;
     }
+
+    push(&b);
 }
 
 /*
@@ -1683,7 +1908,7 @@ f_value(union argument *arg)
     while (p) {
 	if (!strcmp(p->udv_name, a.v.string_val)) {
 	    result = p->udv_value;
-	    if (p->udv_undef)
+	    if (p->udv_value.type == NOTDEFINED)
 		p = NULL;
 	    else if (result.type == STRING)
 		result.v.string_val = gp_strdup(result.v.string_val);
@@ -1696,6 +1921,7 @@ f_value(union argument *arg)
 	/* int_warn(NO_CARET,"undefined variable name passed to value()"); */
 	result.type = CMPLX;
 	result.v.cmplx_val.real = not_a_number();
+	result.v.cmplx_val.imag = 0;
     }
     push(&result);
 }
