@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: parse.c,v 1.88.2.10 2017/08/10 20:42:29 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: parse.c,v 1.112.2.4 2017/08/30 18:37:07 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - parse.c */
@@ -62,8 +62,10 @@ int at_highest_column_used = -1;
 /* This is checked by df_readascii() */
 TBOOLEAN parse_1st_row_as_headers = FALSE;
 
+/* This is used by df_open() and df_readascii() */
+udvt_entry *df_array = NULL;
+
 /* Iteration structures used for bookkeeping */
-/* Iteration can be nested so long as different iterators are used */
 t_iterator * plot_iterator = NULL;
 t_iterator * set_iterator = NULL;
 
@@ -98,6 +100,7 @@ static void parse_multiplicative_expression __PROTO((void));
 static void parse_unary_expression __PROTO((void));
 static void parse_sum_expression __PROTO((void));
 static int  parse_assignment_expression __PROTO((void));
+static int  parse_array_assignment_expression __PROTO((void));
 static int is_builtin_function __PROTO((int t_num));
 
 static void set_up_columnheader_parsing __PROTO((struct at_entry *previous ));
@@ -172,6 +175,13 @@ const_express(struct value *valptr)
     if (undefined) {
 	int_error(tkn, "undefined value");
     }
+
+    if (valptr->type == ARRAY) {
+	/* Make sure no one tries to free it later */
+	valptr->type = NOTDEFINED;
+	int_error(NO_CARET, "const_express: unsupported array operation");
+    }
+
     return (valptr);
 }
 
@@ -190,9 +200,11 @@ string_or_express(struct at_type **atptr)
     int i;
     TBOOLEAN has_dummies;
 
+    static char *array_placeholder = "@@";
     static char* str = NULL;
     free(str);
     str = NULL;
+    df_array = NULL;
 
     if (atptr)
 	*atptr = NULL;
@@ -207,6 +219,14 @@ string_or_express(struct at_type **atptr)
     if (isstring(c_token) && (str = try_to_get_string()))
 	return str;
 
+    /* If this is a bare array name for an existing array, store a pointer */
+    /* for df_open() to use.  "@@" is a magic pseudo-filename passed to    */
+    /* df_open() that tells it to use the stored pointer.                  */
+    if (type_udv(c_token) == ARRAY && !equals(c_token+1, "[")) {
+	df_array = add_udv(c_token++);
+	return array_placeholder;
+    }
+
     /* parse expression */
     temp_at();
 
@@ -421,29 +441,100 @@ accept_multiplicative_expression()
 static int
 parse_assignment_expression()
 {
-    /* Check for assignment operator */
+    /* Check for assignment operator Var = <expr> */
     if (isletter(c_token) && equals(c_token + 1, "=")) {
+
 	/* push the variable name */
 	union argument *foo = add_action(PUSHC);
 	char *varname = NULL;
 	m_capture(&varname,c_token,c_token);
 	foo->v_arg.type = STRING;
 	foo->v_arg.v.string_val = varname;
+
+	/* push a dummy variable that would be the index if this were an array */
+	/* FIXME: It would be nice to hide this from "show at" */
+	foo = add_action(PUSHC);
+	foo->v_arg.type = NOTDEFINED;
+
+	/* push the expression whose value it will get */
 	c_token += 2;
-	/* and the expression whose value it will get */
 	parse_expression();
-	/* and the actual assignment operation */
+
+	/* push the actual assignment operation */
 	(void) add_action(ASSIGN);
 	return 1;
     }
+
+    /* Check for assignment to an array element Array[<expr>] = <expr> */
+    if (parse_array_assignment_expression())
+	return 1;
+
     return 0;
 }
 
+/*
+ * If an array assignment is the first thing on a command line it is handled by
+ * the separate routine array_assignment().
+ * Here we catch assignments that are embedded in an expression.
+ * Examples:
+ *	print A[2] = foo
+ *	A[1] = A[2] = A[3] = 0
+ */
+static int
+parse_array_assignment_expression()
+{
+    /* Check for assignment to an array element Array[<expr>] = <expr> */
+    if (isletter(c_token) && equals(c_token+1, "[")) {
+	char *varname = NULL;
+	union argument *foo;
+	int save_action, save_token;
+
+	/* Quick check for the most common false positives */
+	/* i.e. other constructs that begin with "name["   */
+	if (equals(c_token+3, ":"))
+	    return 0;
+	if (equals(c_token+3, "]") && !equals(c_token+4, "="))
+	    return 0;
+
+	/* Save state of the action table and the command line */
+	save_action = at->a_count;
+	save_token = c_token;
+
+	/* push the array name */
+	m_capture(&varname,c_token,c_token);
+	foo = add_action(PUSHC);
+	foo->v_arg.type = STRING;
+	foo->v_arg.v.string_val = varname;
+
+	/* push the index */
+	c_token += 2;
+	parse_expression();
+
+	/* If this wasn't really an array element assignment, back out */
+	if (!equals(c_token, "]") || !equals(c_token+1, "=")) {
+	    c_token = save_token;
+	    at->a_count = save_action;
+	    free(varname);
+	    return 0;
+	}
+
+	/* Now we evaluate the expression whose value it will get */
+	c_token += 2;
+	parse_expression();
+
+	/* push the actual assignment operation */
+	(void) add_action(ASSIGN);
+	return 1;
+    }
+
+    return 0;
+}
 
 /* add action table entries for primary expressions, i.e. either a
  * parenthesized expression, a variable name, a numeric constant, a
  * function evaluation, a power operator or postfix '!' (factorial)
- * expression */
+ * expression.
+ * Sep 2016 cardinality expression |Array| */
 static void
 parse_primary_expression()
 {
@@ -465,25 +556,49 @@ parse_primary_expression()
 	struct value a;
 
 	c_token++;
-	if (equals(c_token,"N")) {	/* $N == pseudocolumn -3 means "last column" */
+	if (!isanumber(c_token)) {
+	    if (equals(c_token+1, "[")) {
+		struct udvt_entry *datablock_udv;
+		c_token--;
+		datablock_udv = get_udv_by_name(parse_datablock_name());
+		if (!datablock_udv)
+		    int_error(c_token-2,"No such datablock");
+		add_action(PUSH)->udv_arg = datablock_udv;
+	    } else
+		int_error(c_token, "Column number or datablock line expected");
+	} else if (equals(c_token,"N")) {
+	    /* $N == pseudocolumn -3 means "last column" */
 	    c_token++;
 	    Ginteger(&a, -3);
 	    at_highest_column_used = -3;
-	} else if (!isanumber(c_token)) {
-	    int_error(c_token, "Column number expected");
+	    add_action(DOLLARS)->v_arg = a;
 	} else {
 	    convert(&a, c_token++);
 	    if (a.type != INTGR || a.v.int_val < 0)
 		int_error(c_token, "Positive integer expected");
 	    if (at_highest_column_used < a.v.int_val)
 		at_highest_column_used = a.v.int_val;
+	    add_action(DOLLARS)->v_arg = a;
 	}
-	add_action(DOLLARS)->v_arg = a;
+    } else if (equals(c_token, "|")) {
+	struct udvt_entry *udv;
+	c_token++;
+	if (equals(c_token,"$")) {
+	    udv = get_udv_by_name(parse_datablock_name());
+	    if (!udv)
+		int_error(c_token-1, "no such datablock");
+	} else {
+	    udv = add_udv(c_token++);
+	    if (udv->udv_value.type != ARRAY)
+		int_error(c_token-1, "not an array");
+	}
+	add_action(PUSH)->udv_arg = udv;
+	if (!equals(c_token, "|"))
+	    int_error(c_token, "'|' expected");
+	c_token++;
+	add_action(CARDINALITY);
     } else if (isanumber(c_token)) {
-	/* work around HP 9000S/300 HP-UX 9.10 cc limitation ... */
-	/* HBB 20010724: use this code for all platforms, then */
 	union argument *foo = add_action(PUSHC);
-
 	convert(&(foo->v_arg), c_token);
 	c_token++;
     } else if (isletter(c_token)) {
@@ -502,7 +617,10 @@ parse_primary_expression()
 		struct udvt_entry *udv = add_udv(c_token+2);
 		union argument *foo = add_action(PUSHC);
 		foo->v_arg.type = INTGR;
-		foo->v_arg.v.int_val = udv->udv_undef ? 0 : 1;
+		if (udv->udv_value.type == NOTDEFINED)
+		    foo->v_arg.v.int_val = 0;
+		else
+		    foo->v_arg.v.int_val = 1;
 		c_token += 4;  /* skip past "defined ( <foo> ) " */
 		return;
 	    }
@@ -609,50 +727,71 @@ parse_primary_expression()
 	/* this dynamically allocated string will be freed by free_at() */
 	m_quote_capture(&(foo->v_arg.v.string_val), c_token, c_token);
 	c_token++;
-    } else
+    } else {
 	int_error(c_token, "invalid expression ");
-
-    /* add action code for ! (factorial) operator */
-    while (equals(c_token, "!")) {
-	c_token++;
-	(void) add_action(FACTORIAL);
-    }
-    /* add action code for ** operator */
-    if (equals(c_token, "**")) {
-	c_token++;
-	parse_unary_expression();
-	(void) add_action(POWER);
     }
 
-    /* Parse and add actions for range specifier applying to previous entity.
-     * Currently only used to generate substrings, but could also be used to
-     * extract vector slices.
-     */
-    if (equals(c_token, "[") && !isanumber(c_token-1)) {
-	/* handle '*' or empty start of range */
-	if (equals(++c_token,"*") || equals(c_token,":")) {
-	    union argument *empty = add_action(PUSHC);
-	    empty->v_arg.type = INTGR;
-	    empty->v_arg.v.int_val = 1;
-	    if (equals(c_token,"*"))
+    /* The remaining operators are postfixes and can be stacked, e.g. */
+    /* Array[i]**2, so we may have to loop to catch all of them.      */
+    while (TRUE) {
+
+	/* add action code for ! (factorial) operator */
+	if (equals(c_token, "!")) {
+	    c_token++;
+	    (void) add_action(FACTORIAL);
+	}
+
+	/* add action code for ** operator */
+	else if (equals(c_token, "**")) {
+	    c_token++;
+	    parse_unary_expression();
+	    (void) add_action(POWER);
+	}
+
+	/* Parse and add actions for range specifier applying to previous entity.
+	 * Currently the [beg:end] form is used to generate substrings, but could
+	 * also be used to extract vector slices.  The [i] form is used to index
+	 * arrays, but could also be a shorthand for extracting a single-character
+	 * substring.
+	 */
+	else if (equals(c_token, "[") && !isanumber(c_token-1)) {
+	    /* handle '*' or empty start of range */
+	    if (equals(++c_token,"*") || equals(c_token,":")) {
+		union argument *empty = add_action(PUSHC);
+		empty->v_arg.type = INTGR;
+		empty->v_arg.v.int_val = 1;
+		if (equals(c_token,"*"))
+		    c_token++;
+	    } else
+		parse_expression();
+
+	    /* handle array indexing (single value in square brackets) */
+	    if (equals(c_token, "]")) {
 		c_token++;
-	} else
-	    parse_expression();
-	if (!equals(c_token, ":"))
-	    int_error(c_token, "':' expected");
-	/* handle '*' or empty end of range */
-	if (equals(++c_token,"*") || equals(c_token,"]")) {
-	    union argument *empty = add_action(PUSHC);
-	    empty->v_arg.type = INTGR;
-	    empty->v_arg.v.int_val = 65535; /* should be INT_MAX */
-	    if (equals(c_token,"*"))
-		c_token++;
-	} else
-	    parse_expression();
-	if (!equals(c_token, "]"))
-	    int_error(c_token, "']' expected");
-	c_token++;
-	(void) add_action(RANGE);
+		(void) add_action(INDEX);
+		continue;
+	    }
+
+	    if (!equals(c_token, ":"))
+		int_error(c_token, "':' expected");
+	    /* handle '*' or empty end of range */
+	    if (equals(++c_token,"*") || equals(c_token,"]")) {
+		union argument *empty = add_action(PUSHC);
+		empty->v_arg.type = INTGR;
+		empty->v_arg.v.int_val = 65535; /* should be INT_MAX */
+		if (equals(c_token,"*"))
+		    c_token++;
+	    } else
+		parse_expression();
+	    if (!equals(c_token, "]"))
+		int_error(c_token, "']' expected");
+	    c_token++;
+	    (void) add_action(RANGE);
+
+	/* Whatever this is, it isn't another postfix operator */
+	} else {
+	    break;
+	}
     }
 }
 
@@ -959,8 +1098,6 @@ parse_link_via( struct udft_entry *udf )
 	int_error(c_token,"Missing expression");
 
     /* Save action table for the linkage mapping */
-    strcpy(c_dummy_var[0], "x");
-    strcpy(c_dummy_var[1], "y");
     dummy_func = udf;
     free_at(udf->at);
     udf->at = perm_at();
@@ -1059,6 +1196,8 @@ add_udv(int t_num)
 {
     char varname[MAX_ID_LEN+1];
     copy_str(varname, t_num, MAX_ID_LEN);
+    if (token[t_num].length > MAX_ID_LEN-1)
+	int_warn(t_num, "truncating variable name that is too long");
     return add_udv_by_name(varname);
 }
 
@@ -1150,11 +1289,13 @@ check_for_iteration()
     t_iterator *iter = NULL;
     t_iterator *prev = NULL;
     t_iterator *this_iter = NULL;
+    TBOOLEAN no_parent = FALSE;
 
     /* Now checking for iteration parameters */
     /* Nested "for" statements are supported, each one corresponds to a node of the linked list */
     while (equals(c_token, "for")) {
 	struct udvt_entry *iteration_udv = NULL;
+	t_value original_udv_value;
 	char *iteration_string = NULL;
 	int iteration_start;
 	int iteration_end;
@@ -1168,6 +1309,8 @@ check_for_iteration()
 	if (!equals(c_token++, "[") || !isletter(c_token))
 	    int_error(c_token-1, errormsg);
 	iteration_udv = add_udv(c_token++);
+	original_udv_value = iteration_udv->udv_value;
+	iteration_udv->udv_value.type = NOTDEFINED;
 
 	if (equals(c_token, "=")) {
 	    c_token++;
@@ -1178,8 +1321,12 @@ check_for_iteration()
 		/* Save the expression as well as the value */
 		struct value v;
 		iteration_start_at = perm_at();
-		evaluate_at(iteration_start_at, &v);
-		iteration_start = real(&v);
+		if (no_parent) {
+		    iteration_start = 0;
+		} else {
+		    evaluate_at(iteration_start_at, &v);
+		    iteration_start = real(&v);
+		}
 	    }
 	    if (!equals(c_token++, ":"))
 	    	int_error(c_token-1, errormsg);
@@ -1193,8 +1340,12 @@ check_for_iteration()
 		/* Save the expression as well as the value */
 		struct value v;
 		iteration_end_at = perm_at();
-		evaluate_at(iteration_end_at, &v);
-		iteration_end = real(&v);
+		if (no_parent) {
+		    iteration_end = 0;
+		} else {
+		    evaluate_at(iteration_end_at, &v);
+		    iteration_end = real(&v);
+		}
 	    }
 	    if (equals(c_token,":")) {
 	    	c_token++;
@@ -1204,10 +1355,9 @@ check_for_iteration()
 	    }
 	    if (!equals(c_token++, "]"))
 	    	int_error(c_token-1, errormsg);
-	    if (iteration_udv->udv_undef == FALSE)
-		gpfree_string(&(iteration_udv->udv_value));
+	    gpfree_array(&(iteration_udv->udv_value));
+	    gpfree_string(&(iteration_udv->udv_value));
 	    Ginteger(&(iteration_udv->udv_value), iteration_start);
-	    iteration_udv->udv_undef = FALSE;
 	}
 	else if (equals(c_token++, "in")) {
 	    /* Assume this is a string-valued expression. */
@@ -1222,10 +1372,9 @@ check_for_iteration()
 	    iteration_string = v.v.string_val;
 	    iteration_start = 1;
 	    iteration_end = gp_words(iteration_string);
-	    if (iteration_udv->udv_undef == FALSE)
-	    	gpfree_string(&(iteration_udv->udv_value));
+	    gpfree_array(&(iteration_udv->udv_value));
+	    gpfree_string(&(iteration_udv->udv_value));
 	    Gstring(&(iteration_udv->udv_value), gp_word(iteration_string, 1));
-	    iteration_udv->udv_undef = FALSE;
 	}
 	else /* Neither [i=B:E] or [s in "foo"] */
 	    int_error(c_token-1, errormsg);
@@ -1233,6 +1382,7 @@ check_for_iteration()
 	iteration_current = iteration_start;
 
 	this_iter = gp_alloc(sizeof(t_iterator), "iteration linked list");
+	this_iter->original_udv_value = original_udv_value;
 	this_iter->iteration_udv = iteration_udv; 
 	this_iter->iteration_string = iteration_string;
 	this_iter->iteration_start = iteration_start;
@@ -1248,11 +1398,20 @@ check_for_iteration()
 	    /* first "for" statement: this will be the listhead */
 	    iter = this_iter;
 	} else {
-	    /* not the first "for" statement: attach the newly created node to the end of the list */
+	    /* nested "for": attach newly created node to the end of the list */
 	    prev->next = this_iter;
 	}
 	prev = this_iter;
 
+	/* If some depth of a nested iteration evaluates to an empty range, the
+	 * evaluated limits of depths below it are moot (and possibly invalid).
+	 * This flag tells us to skip their evaluation to avoid irrelevant errors.
+	 */
+	if (no_iteration(this_iter)) {
+	    no_parent = TRUE;
+	    FPRINTF((stderr,"iteration at level %d is moot\n", nesting_depth));
+	}
+
 	nesting_depth++;
     }
 
@@ -1407,6 +1566,8 @@ cleanup_iteration(t_iterator *iter)
 {
     while (iter) {
 	t_iterator *next = iter->next;
+	gpfree_string(&(iter->iteration_udv->udv_value));
+	iter->iteration_udv->udv_value = iter->original_udv_value;
 	free(iter->iteration_string);
 	free_at(iter->start_at);
 	free_at(iter->end_at);
