@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.85.2.4 2013/10/14 23:54:28 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.117 2017/02/17 23:55:53 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - pm3d.c */
@@ -19,9 +19,8 @@ static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.85.2.4 2013/10/14 23:54:28
 #endif
 #include "pm3d.h"
 #include "alloc.h"
-#include "axis.h"
+#include "getcolor.h"		/* for rgb1_from_gray */
 #include "graphics.h"
-#include "graph3d.h"
 #include "hidden3d.h"		/* p_vertex & map3d_xyz() */
 #include "plot2d.h"
 #include "plot3d.h"
@@ -31,8 +30,12 @@ static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.85.2.4 2013/10/14 23:54:28
 
 #include <stdlib.h> /* qsort() */
 
-/* Needed by routine filled_quadrangle() in color.c */
+/* Used to initialize `set pm3d border` */
+struct lp_style_type default_pm3d_border = DEFAULT_LP_STYLE_TYPE;
+
+/* Used by routine filled_quadrangle() in color.c */
 struct lp_style_type pm3d_border_lp;
+TBOOLEAN track_pm3d_quadrangles;
 
 /*
   Global options for pm3d algorithm (to be accessed by set / show).
@@ -44,13 +47,15 @@ pm3d_struct pm3d = {
     0,				/* no flushing triangles */
     PM3D_SCANS_AUTOMATIC,	/* scans direction is determined automatically */
     PM3D_CLIP_4IN,		/* clipping: all 4 points in ranges */
-    0,				/* no pm3d hidden3d is drawn */
     PM3D_EXPLICIT,		/* implicit */
     PM3D_WHICHCORNER_MEAN,	/* color from which corner(s) */
     1,				/* interpolate along scanline */
-    1				/* interpolate between scanlines */
+    1,				/* interpolate between scanlines */
+    DEFAULT_LP_STYLE_TYPE	/* for the border */
 };
 
+lighting_model pm3d_shade;
+
 typedef struct {
     double gray;
     double z; /* maximal z value after rotation to graph coordinate system */
@@ -61,6 +66,8 @@ typedef struct {
     t_colorspec *border_color;	/* Only used by depthorder processing */
 } quadrangle;
 
+#define PM3D_USE_BORDER_COLOR_INSTEAD_OF_GRAY -12345
+
 static int allocated_quadrangles = 0;
 static int current_quadrangle = 0;
 static quadrangle* quadrangles = (quadrangle*)0;
@@ -68,41 +75,48 @@ static quadrangle* quadrangles = (quadrangle*)0;
 /* Internal prototypes for this module */
 static TBOOLEAN plot_has_palette;
 static double geomean4 __PROTO((double, double, double, double));
+static double harmean4 __PROTO((double, double, double, double));
 static double median4 __PROTO((double, double, double, double));
+static double rms4 __PROTO((double, double, double, double));
 static void pm3d_plot __PROTO((struct surface_points *, int));
 static void pm3d_option_at_error __PROTO((void));
 static void pm3d_rearrange_part __PROTO((struct iso_curve *, const int, struct iso_curve ***, int *));
-static void filled_color_contour_plot  __PROTO((struct surface_points *, int));
+static int apply_lighting_model __PROTO(( struct coordinate *, struct coordinate *, struct coordinate *, struct coordinate *, double gray ));
+
 static TBOOLEAN color_from_rgbvar = FALSE;
+static double light[3];
 
 /*
  * Utility routines.
  */
 
 /* Geometrical mean = pow( prod(x_i > 0) x_i, 1/N )
- * Sign of the result: result is positive if 3 or 4 x_i are positive,
- * it is negative if 3 or all 4 x_i are negative. Helps to splot surface
- * with all color coordinates negative.
+ * In order to facilitate plotting surfaces with all color coords negative,
+ * All 4 corners positive - return positive geometric mean
+ * All 4 corners negative - return negative geometric mean
+ * otherwise return 0
+ * EAM Oct 2012: This is a CHANGE
  */
 static double
 geomean4 (double x1, double x2, double x3, double x4)
 {
-    /* honor signess, i.e. sign(geomean) = sign(prod(x_i)) */
     int neg = (x1 < 0) + (x2 < 0) + (x3 < 0) + (x4 < 0);
-    x1 *= x2 * x3 * x4;
-    if (x1 == 0) return 0;
-    /* pow(x, 0.25) is slightly faster than sqrt(sqrt(x)) */
-    x1 = sqrt(sqrt(fabs(x1)));
-#if 0
-    /* such a warning could be helpful, but under normal usage it is just an overhead */
-    if (neg > 1 && interactive && notwarned) {
-	    int notwarned = 1;  ... to be set on every new splot
-	    if (notwarned)
-		int_warn(NO_CARET, "corners2color geomean with negative data points");
-	    notwarned = 0;
-    }
-#endif
-    return (neg <= 2) ? x1 : -x1;
+    double product = x1 * x2 * x3 * x4;
+
+    if (product == 0) return 0;
+    if (neg == 1 || neg == 2 || neg == 3) return 0;
+
+    product = sqrt(sqrt(fabs(product)));
+    return (neg == 0) ? product : -product;
+}
+
+static double
+harmean4 (double x1, double x2, double x3, double x4)
+{
+    if (x1 <= 0 || x2 <= 0 || x3 <= 0 || x4 <= 0)
+	return not_a_number();
+    else
+	return 4 / ((1/x1) + (1/x2) + (1/x3) + (1/x4));
 }
 
 
@@ -144,6 +158,12 @@ maximum4 (double x1, double x2, double x3, double x4)
     return GPMAX(x1, x3);
 }
 
+/* The root mean square of the 4 numbers */
+static double
+rms4 (double x1, double x2, double x3, double x4)
+{
+	return 0.5*sqrt(x1*x1 + x2*x2 + x3*x3 + x4*x4);
+}
 
 /*
 * Now the routines which are really just those for pm3d.c
@@ -153,21 +173,24 @@ maximum4 (double x1, double x2, double x3, double x4)
  * Rescale z to cb values. Nothing to do if both z and cb are linear or log of the
  * same base, other it has to un-log z and subsequently log it again.
  */
+#if defined(NONLINEAR_AXES) && (NONLINEAR_AXES > 0)
+    /* z2cb is a no-op */
+#else
 double
-z2cb(double z)
+z2cb_with_logs(double z)
 {
     if (!Z_AXIS.log && !CB_AXIS.log) /* both are linear */
 	return z;
     if (Z_AXIS.log && !CB_AXIS.log) /* log z, linear cb */
-	return exp(z * Z_AXIS.log_base); /* unlog(z) */
+	return axis_undo_log((&Z_AXIS), z);
     if (!Z_AXIS.log && CB_AXIS.log) /* linear z, log cb */
-	return (z<=0) ? CB_AXIS.min : (log(z) / CB_AXIS.log_base);
+	return (z<=0) ? CB_AXIS.min : axis_do_log((&CB_AXIS), z);
     /* both are log */
     if (Z_AXIS.base==CB_AXIS.base) /* can we compare double numbers like that? */
 	return z;
     return z * Z_AXIS.log_base / CB_AXIS.log_base; /* log_cb(unlog_z(z)) */
 }
-
+#endif
 
 /*
  * Rescale cb (color) value into the interval of grays [0,1], taking care
@@ -177,12 +200,19 @@ z2cb(double z)
 double
 cb2gray(double cb)
 {
-    if (cb <= CB_AXIS.min)
+    AXIS *cbaxis = &CB_AXIS;
+
+    if (cb <= cbaxis->min)
 	return (sm_palette.positive == SMPAL_POSITIVE) ? 0 : 1;
-    if (cb >= CB_AXIS.max)
+    if (cb >= cbaxis->max)
 	return (sm_palette.positive == SMPAL_POSITIVE) ? 1 : 0;
-    cb = (cb - CB_AXIS.min)
-      / (CB_AXIS.max - CB_AXIS.min);
+
+    if (nonlinear(cbaxis)) {
+	cbaxis = cbaxis->linked_to_primary;
+	cb = eval_link_function(cbaxis, cb);
+    }
+
+    cb = (cb - cbaxis->min) / (cbaxis->max - cbaxis->min);
     return (sm_palette.positive == SMPAL_POSITIVE) ? cb : 1-cb;
 }
 
@@ -343,28 +373,26 @@ static int compare_quadrangles(const void* v1, const void* v2)
 
 void pm3d_depth_queue_clear(void)
 {
-    if (pm3d.direction != PM3D_DEPTH)
-	return;
-
-    if (quadrangles)
-	free(quadrangles);
-    quadrangles = (quadrangle*)0;
+    free(quadrangles);
+    quadrangles = NULL;
     allocated_quadrangles = 0;
     current_quadrangle = 0;
 }
 
 void pm3d_depth_queue_flush(void)
 {
-    if (pm3d.direction != PM3D_DEPTH)
+    if (pm3d.direction != PM3D_DEPTH && !track_pm3d_quadrangles)
 	return;
 
+    term->layer(TERM_LAYER_BEGIN_PM3D_FLUSH);
+
     if (current_quadrangle > 0 && quadrangles) {
 
 	quadrangle* qp;
 	quadrangle* qe;
 	gpdPoint* gpdPtr;
 #ifdef EXTENDED_COLOR_SPECS
-	gpdPoint* gpiPtr;
+	gpiPoint* gpiPtr;
 	double w = trans_mat[3][3];
 #endif
 	vertex out;
@@ -399,12 +427,14 @@ void pm3d_depth_queue_flush(void)
 
 	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
 
-	    if (color_from_rgbvar)
-		set_rgbcolor(qp->gray);
+	    /* set the color */
+	    if (qp->gray == PM3D_USE_BORDER_COLOR_INSTEAD_OF_GRAY)
+		apply_pm3dcolor(qp->border_color);
+	    else if (color_from_rgbvar || pm3d_shade.strength > 0)
+		set_rgbcolor_var((unsigned int)qp->gray);
 	    else
 		set_color(qp->gray);
-	    if (pm3d.hidden3d_tag < 0)
-		pm3d_border_lp.pm3d_color = *(qp->border_color);
+
 #ifdef EXTENDED_COLOR_SPECS
 	    ifilled_quadrangle(qp->icorners);
 #else
@@ -414,25 +444,24 @@ void pm3d_depth_queue_flush(void)
     }
 
     pm3d_depth_queue_clear();
+
+    term->layer(TERM_LAYER_END_PM3D_FLUSH);
 }
 
 /*
  * Now the implementation of the pm3d (s)plotting mode
- *
- * Note: the input parameter at_which_z is char, but an old HP cc requires
- * ANSI C K&R routines with int only.
  */
 static void
 pm3d_plot(struct surface_points *this_plot, int at_which_z)
 {
-    int j, i, i1, ii, ii1, from, curve, scan, up_to, up_to_minus, invert = 0;
+    int j, i, i1, ii, ii1, from, scan, up_to, up_to_minus, invert = 0;
     int go_over_pts, max_pts;
     int are_ftriangles, ftriangles_low_pt = -999, ftriangles_high_pt = -999;
     struct iso_curve *scanA, *scanB;
     struct coordinate GPHUGE *pointsA, *pointsB;
     struct iso_curve **scan_array;
     int scan_array_n;
-    double avgC, gray;
+    double avgC, gray = 0;
     double cb1, cb2, cb3, cb4;
     gpdPoint corners[4];
     int interp_i, interp_j;
@@ -443,7 +472,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
     /* just a shortcut */
     TBOOLEAN color_from_column = this_plot->pm3d_color_from_column;
-    
+
     color_from_rgbvar = (this_plot->lp_properties.pm3d_color.type == TC_RGB
 			&&  this_plot->lp_properties.pm3d_color.value == -1);
 
@@ -452,8 +481,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
     /* Apply and save the user-requested line properties */
     pm3d_border_lp = this_plot->lp_properties;
-    if (pm3d.hidden3d_tag > 0)
-	lp_use_properties(&pm3d_border_lp, pm3d.hidden3d_tag);
     term_apply_lp_properties(&pm3d_border_lp);
 
     if (at_which_z != PM3D_AT_BASE && at_which_z != PM3D_AT_TOP && at_which_z != PM3D_AT_SURFACE)
@@ -474,7 +501,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     }
 
     scanA = this_plot->iso_crvs;
-    curve = 0;
 
     pm3d_rearrange_scan_array(this_plot, &scan_array, &scan_array_n, &invert, (struct iso_curve ***) 0, (int *) 0, (int *) 0);
 
@@ -506,6 +532,9 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    }
 	}
 
+	if (max_scan_pts == 0 || max_scans == 0)
+	    int_error(NO_CARET, "all scans empty");
+
 	if (interp_i <= 0) {
 	    ii = (interp_i == 0) ? DEFAULT_OPTIMAL_NB_POINTS : -interp_i;
 	    interp_i = floor(ii / max_scan_pts) + 1;
@@ -523,6 +552,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     }
 
     if (pm3d.direction == PM3D_DEPTH) {
+	int needed_quadrangles = 0;
 
 	for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
 
@@ -531,15 +561,21 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
 	    are_ftriangles = pm3d.ftriangles && (scanA->p_count != scanB->p_count);
 	    if (!are_ftriangles)
-		allocated_quadrangles += GPMIN(scanA->p_count, scanB->p_count) - 1;
+		needed_quadrangles += GPMIN(scanA->p_count, scanB->p_count) - 1;
 	    else {
-		allocated_quadrangles += GPMAX(scanA->p_count, scanB->p_count) - 1;
+		needed_quadrangles += GPMAX(scanA->p_count, scanB->p_count) - 1;
 	    }
 	}
-	allocated_quadrangles *= (interp_i > 1) ? interp_i : 1;
-	allocated_quadrangles *= (interp_j > 1) ? interp_j : 1;
-	quadrangles = (quadrangle*)gp_realloc(quadrangles, allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
-	/* DEBUG: fprintf(stderr, "allocated_quadrangles = %d\n", allocated_quadrangles); */
+	needed_quadrangles *= (interp_i > 1) ? interp_i : 1;
+	needed_quadrangles *= (interp_j > 1) ? interp_j : 1;
+
+	while (current_quadrangle + needed_quadrangles >= allocated_quadrangles) {
+	    FPRINTF((stderr, "allocated_quadrangles = %d current = %d needed = %d\n",
+		allocated_quadrangles, current_quadrangle, needed_quadrangles));
+	    allocated_quadrangles = needed_quadrangles + 2*allocated_quadrangles;	
+	    quadrangles = (quadrangle*)gp_realloc(quadrangles, 
+			allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
+	}
     }
     /* pm3d_rearrange_scan_array(this_plot, (struct iso_curve***)0, (int*)0, &scan_array, &invert); */
 
@@ -643,7 +679,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    }
 	    /* From here, i is index to scan A, ii to scan B */
 	    if (scanA->p_count > scanB->p_count) {
-	        int itmp = i;
+		int itmp = i;
 		i = ii;
 		ii = itmp;
 		itmp = i1;
@@ -672,14 +708,16 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    }
 
 	    if ((interp_i <= 1 && interp_j <= 1) || pm3d.direction == PM3D_DEPTH) {
+
 #ifdef EXTENDED_COLOR_SPECS
-	      if (!supply_extended_color_specs) {
+	      if ((term->flags & TERM_EXTENDED_COLOR) == 0)
 #endif
+	      {
 		/* Get the gray as the average of the corner z- or gray-positions
 		   (note: log scale is already included). The average is calculated here
 		   if there is no interpolation (including the "pm3d depthorder" option),
 		   otherwise it is done for each interpolated quadrangle later.
-		   I always wonder what is faster: d*0.25 or d/4? Someone knows? -- 0.25 (joze) */
+		 */
 		if (color_from_column) {
 		    /* color is set in plot3d.c:get_3ddata() */
 		    cb1 = pointsA[i].CRD_COLOR;
@@ -692,51 +730,79 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		    cb3 = z2cb(pointsB[ii].z);
 		    cb4 = z2cb(pointsB[ii1].z);
 		}
-		switch (pm3d.which_corner_color) {
-		    default:
-		    case PM3D_WHICHCORNER_MEAN:
-			if (color_from_rgbvar) {
-			    int r = (((int)cb1)&0xff0000)+(((int)cb2)&0xff0000)
-			    	  + (((int)cb3)&0xff0000)+(((int)cb4)&0xff0000);
-			    int g = (((int)cb1)&0xff00)+(((int)cb2)&0xff00)
-			    	  + (((int)cb3)&0xff00)+(((int)cb4)&0xff00);
-			    int b = (((int)cb1)&0xff)+(((int)cb2)&0xff)
-			    	  + (((int)cb3)&0xff)+(((int)cb4)&0xff);
+		/* Fancy averages of RGB color make no sense */
+		if (color_from_rgbvar) {
+		    unsigned int r, g, b, a;
+		    unsigned int u1 = cb1;
+		    unsigned int u2 = cb2;
+		    unsigned int u3 = cb3;
+		    unsigned int u4 = cb4;
+		    switch (pm3d.which_corner_color) {
+			default:
+			    r = (u1&0xff0000) + (u2&0xff0000) + (u3&0xff0000) + (u4&0xff0000);
+			    g = (u1&0xff00) + (u2&0xff00) + (u3&0xff00) + (u4&0xff00);
+			    b = (u1&0xff) + (u2&0xff) + (u3&0xff) + (u4&0xff);
 			    avgC = ((r>>2)&0xff0000) + ((g>>2)&0xff00) + ((b>>2)&0xff);
-			} else {
-    			    avgC = (cb1 + cb2 + cb3 + cb4) * 0.25;
-			}
-			break;
-		    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
-		    case PM3D_WHICHCORNER_C1: avgC = cb1; break;
-		    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
-		    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
-		    case PM3D_WHICHCORNER_C4: avgC = cb4; break;
+			    a = ((u1>>24)&0xff) + ((u2>>24)&0xff) + ((u3>>24)&0xff) + ((u4>>24)&0xff);
+			    avgC += (a<<22)&0xff000000;
+			    break;
+			case PM3D_WHICHCORNER_C1: avgC = cb1; break;
+			case PM3D_WHICHCORNER_C2: avgC = cb2; break;
+			case PM3D_WHICHCORNER_C3: avgC = cb3; break;
+			case PM3D_WHICHCORNER_C4: avgC = cb4; break;
+		    }
+
+		/* But many different averages are possible for gray values */
+		} else  {
+		    switch (pm3d.which_corner_color) {
+			default:
+			case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
+			case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_HARMEAN: avgC = harmean4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_RMS: avgC = rms4(cb1, cb2, cb3, cb4); break;
+			case PM3D_WHICHCORNER_C1: avgC = cb1; break;
+			case PM3D_WHICHCORNER_C2: avgC = cb2; break;
+			case PM3D_WHICHCORNER_C3: avgC = cb3; break;
+			case PM3D_WHICHCORNER_C4: avgC = cb4; break;
+		    }
 		}
 
-		if (color_from_rgbvar) /* we were given an explicit color */
+		/* The value is out of range, but we didn't figure it out until now */
+		if (isnan(avgC))
+		    continue;
+
+		if (color_from_rgbvar) /* we were given an RGB color */
 			gray = avgC;
 		else /* transform z value to gray, i.e. to interval [0,1] */
 			gray = cb2gray(avgC);
 
-		/* print the quadrangle with the given color */
-		FPRINTF((stderr, "averageColor %g\tgray=%g\tM %g %g L %g %g L %g %g L %g %g\n",
-		       avgC, gray, pointsA[i].x, pointsA[i].y, pointsB[ii].x, pointsB[ii].y,
-		       pointsB[ii1].x, pointsB[ii1].y, pointsA[i1].x, pointsA[i1].y));
+		/* apply lighting model */
+		if (pm3d_shade.strength > 0) {
+		    if (at_which_z == PM3D_AT_SURFACE)
+			gray = apply_lighting_model( &pointsA[i], &pointsA[i1],
+					&pointsB[ii], &pointsB[ii1], gray);
+		    /* Don't apply lighting model to TOP/BOTTOM projections  */
+		    /* but convert from floating point 0<gray<1 to RGB color */
+		    /* since that is what would have been returned from the  */
+		    /* lighting code.					     */
+		    else if (!color_from_rgbvar) {
+			rgb255_color temp;
+			rgb255maxcolors_from_gray(gray, &temp);
+			gray = (long)((temp.r << 16) + (temp.g << 8) + (temp.b));
+		    }
+		}
 
 		/* set the color */
 		if (pm3d.direction != PM3D_DEPTH) {
-		    if (color_from_rgbvar)
-			set_rgbcolor(gray);
+		    if (color_from_rgbvar || pm3d_shade.strength > 0)
+			set_rgbcolor_var((unsigned int)gray);
 		    else
 			set_color(gray);
 		}
-#ifdef EXTENDED_COLOR_SPECS
 	      }
-#endif
 	    }
 
 	    corners[0].x = pointsA[i].x;
@@ -749,9 +815,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    corners[3].y = pointsA[i1].y;
 
 	    if (interp_i > 1 || interp_j > 1 || at_which_z == PM3D_AT_SURFACE) {
-		/* always supply the z value if
-		 * EXTENDED_COLOR_SPECS is defined
-		 */
 		corners[0].z = pointsA[i].z;
 		corners[1].z = pointsB[ii].z;
 		corners[2].z = pointsB[ii1].z;
@@ -764,7 +827,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		}
 	    }
 #ifdef EXTENDED_COLOR_SPECS
-	    if (supply_extended_color_specs) {
+	    if ((term->flags & TERM_EXTENDED_COLOR)) {
 		if (color_from_column) {
 		    icorners[0].z = pointsA[i].CRD_COLOR;
 		    icorners[1].z = pointsB[ii].CRD_COLOR;
@@ -785,6 +848,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    if (pm3d.direction == PM3D_DEPTH) {
 		/* copy quadrangle */
 		quadrangle* qp = quadrangles + current_quadrangle;
+
 		memcpy(qp->corners, corners, 4 * sizeof (gpdPoint));
 		qp->gray = gray;
 		for (i = 0; i < 4; i++) {
@@ -804,9 +868,9 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		 * defined by corners[3],corners[0] and corners[2],corners[1]. */
 		int j1;
 		for (i1 = 0; i1 <= interp_i; i1++) {
-		    bl_point[i1][0].x = 
+		    bl_point[i1][0].x =
 			((corners[3].x - corners[0].x) / interp_i) * i1 + corners[0].x;
-		    bl_point[i1][interp_j].x = 
+		    bl_point[i1][interp_j].x =
 			((corners[2].x - corners[1].x) / interp_i) * i1 + corners[1].x;
 		    bl_point[i1][0].y =
 			((corners[3].y - corners[0].y) / interp_i) * i1 + corners[0].y;
@@ -870,28 +934,30 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			    corners[2].x, corners[2].y, corners[3].x, corners[3].y));
 
 			/* If the colors are given separately, we already loaded them above */
-			if (!color_from_column) {
-			    cb1 = z2cb(corners[0].z);
-			    cb2 = z2cb(corners[1].z);
-			    cb3 = z2cb(corners[2].z);
-			    cb4 = z2cb(corners[3].z);
-			} else {
+			if (color_from_column) {
 			    cb1 = corners[0].c;
 			    cb2 = corners[1].c;
 			    cb3 = corners[2].c;
 			    cb4 = corners[3].c;
+			} else {
+			    cb1 = z2cb(corners[0].z);
+			    cb2 = z2cb(corners[1].z);
+			    cb3 = z2cb(corners[2].z);
+			    cb4 = z2cb(corners[3].z);
 			}
 			switch (pm3d.which_corner_color) {
+			    default:
 			    case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
 			    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_HARMEAN: avgC = harmean4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_MAX: avgC = maximum4(cb1, cb2, cb3, cb4); break;
+			    case PM3D_WHICHCORNER_RMS: avgC = rms4(cb1, cb2, cb3, cb4); break;
 			    case PM3D_WHICHCORNER_C1: avgC = cb1; break;
 			    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
 			    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
 			    case PM3D_WHICHCORNER_C4: avgC = cb4; break;
-			    default: int_error(NO_CARET, "cannot be here"); avgC = 0;
 			}
 
 			if (color_from_rgbvar) /* we were given an explicit color */
@@ -899,6 +965,31 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			else /* transform z value to gray, i.e. to interval [0,1] */
 				gray = cb2gray(avgC);
 
+			/* apply lighting model */
+			if (pm3d_shade.strength > 0) {
+			    /* FIXME: coordinate->quadrangle->coordinate seems crazy */
+			    coordinate corcorners[4];
+			    int i;
+			    for (i=0; i<4; i++) {
+				corcorners[i].x = corners[i].x;
+				corcorners[i].y = corners[i].y;
+				corcorners[i].z = corners[i].z;
+			    }
+
+			    if (at_which_z == PM3D_AT_SURFACE)
+				gray = apply_lighting_model( &corcorners[0], &corcorners[1],
+						&corcorners[2], &corcorners[3], gray);
+			    /* Don't apply lighting model to TOP/BOTTOM projections  */
+			    /* but convert from floating point 0<gray<1 to RGB color */
+			    /* since that is what would have been returned from the  */
+			    /* lighting code.					     */
+			    else if (!color_from_rgbvar) {
+				rgb255_color temp;
+				rgb255maxcolors_from_gray(gray, &temp);
+				gray = (long)((temp.r << 16) + (temp.g << 8) + (temp.b));
+			    }
+			}
+	
 			if (pm3d.direction == PM3D_DEPTH) {
 			    /* copy quadrangle */
 			    quadrangle* qp = quadrangles + current_quadrangle;
@@ -907,8 +998,8 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			    qp->border_color = &this_plot->lp_properties.pm3d_color;
 			    current_quadrangle++;
 			} else {
-			    if (color_from_rgbvar)
-				set_rgbcolor(gray);
+			    if (pm3d_shade.strength > 0 || color_from_rgbvar)
+				set_rgbcolor_var((unsigned int)gray);
 			    else
 				set_color(gray);
 			    if (at_which_z == PM3D_AT_BASE)
@@ -944,7 +1035,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     free(scan_array);
 }				/* end of pm3d splotting mode */
 
-
+#ifdef PM3D_CONTOURS
 /*
  *  Now the implementation of the filled color contour plot
 */
@@ -992,6 +1083,7 @@ filled_color_contour_plot(struct surface_points *this_plot, int contours_where)
 	cntr = cntr->next;
     }
 }				/* end of filled color contour plot splot mode */
+#endif
 
 
 /*
@@ -1005,11 +1097,15 @@ pm3d_reset()
     pm3d.ftriangles = 0;
     pm3d.direction = PM3D_SCANS_AUTOMATIC;
     pm3d.clip = PM3D_CLIP_4IN;
-    pm3d.hidden3d_tag = 0;
     pm3d.implicit = PM3D_EXPLICIT;
     pm3d.which_corner_color = PM3D_WHICHCORNER_MEAN;
     pm3d.interp_i = 1;
     pm3d.interp_j = 1;
+    pm3d.border.l_type = LT_NODRAW;
+
+    pm3d_shade.strength = 0.0;
+    pm3d_shade.spec = 0.0;
+    pm3d_shade.fixed = TRUE;
 }
 
 
@@ -1027,6 +1123,13 @@ pm3d_draw_one(struct surface_points *plot)
     if (!where[0])
 	return;
 
+    /* Initialize lighting model */
+    if (pm3d_shade.strength > 0) {
+	light[0] = cos(-DEG2RAD*pm3d_shade.rot_x)*cos(-(DEG2RAD*pm3d_shade.rot_z+90));
+	light[2] = cos(-DEG2RAD*pm3d_shade.rot_x)*sin(-(DEG2RAD*pm3d_shade.rot_z+90));
+	light[1] = sin(-DEG2RAD*pm3d_shade.rot_x);
+    }
+
     /* for pm3dCompress.awk */
     if (pm3d.direction != PM3D_DEPTH)
 	term->layer(TERM_LAYER_BEGIN_PM3D_MAP);
@@ -1035,6 +1138,7 @@ pm3d_draw_one(struct surface_points *plot)
 	pm3d_plot(plot, where[i]);
     }
 
+#ifdef PM3D_CONTOURS
     if (strchr(where, 'C') != NULL) {
 	/* !!!!! FILLED COLOR CONTOURS, *UNDOCUMENTED*
 	   !!!!! LATER CHANGE TO STH LIKE
@@ -1048,6 +1152,7 @@ pm3d_draw_one(struct surface_points *plot)
 	if (draw_contour & CONTOUR_BASE)
 	    filled_color_contour_plot(plot, CONTOUR_BASE);
     }
+#endif
 
     /* for pm3dCompress.awk */
     if (pm3d.direction != PM3D_DEPTH)
@@ -1055,6 +1160,25 @@ pm3d_draw_one(struct surface_points *plot)
 }
 
 
+/*
+ * Add one pm3d quadrangle to the mix.
+ * Called by zerrorfill().
+ */
+void
+pm3d_add_quadrangle(struct surface_points *plot, gpdPoint corners[4])
+{
+    quadrangle *q;
+
+    if (allocated_quadrangles < current_quadrangle + plot->iso_crvs->p_count) {
+	allocated_quadrangles += 2 * plot->iso_crvs->p_count;
+	quadrangles = (quadrangle*)gp_realloc(quadrangles, allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
+    }
+    q = quadrangles + current_quadrangle++;
+    memcpy(q->corners, corners, 4*sizeof(gpdPoint));
+    q->border_color = &plot->fill_properties.border_color;
+    q->gray = PM3D_USE_BORDER_COLOR_INSTEAD_OF_GRAY;
+}
+
 /* Display an error message for the routine get_pm3d_at_option() below.
  */
 
@@ -1107,6 +1231,7 @@ set_plot_with_palette(int plot_num, int plot_mode)
     struct curve_points *this_2dplot = first_plot;
     int surface = 0;
     struct text_label *this_label = first_label;
+    struct object *this_object;
 
     plot_has_palette = TRUE;
     /* Is pm3d switched on globally? */
@@ -1118,10 +1243,9 @@ set_plot_with_palette(int plot_num, int plot_mode)
 	while (this_2dplot) {
 	    if (this_2dplot->plot_style == IMAGE)
 		return;
-	    if (this_2dplot->lp_properties.use_palette
-	    && (this_2dplot->lp_properties.pm3d_color.type == TC_CB
+	    if (this_2dplot->lp_properties.pm3d_color.type == TC_CB
 	    ||  this_2dplot->lp_properties.pm3d_color.type == TC_FRAC
-	    ||  this_2dplot->lp_properties.pm3d_color.type == TC_Z))
+	    ||  this_2dplot->lp_properties.pm3d_color.type == TC_Z)
 		return;
 	    if (this_2dplot->labels
 	    && (this_2dplot->labels->textcolor.type == TC_CB
@@ -1136,19 +1260,19 @@ set_plot_with_palette(int plot_num, int plot_mode)
     if (plot_mode == MODE_SPLOT) {
 	/* Any surface 'with pm3d', 'with image' or 'with line|dot palette'? */
 	while (surface++ < plot_num) {
+	    int type;
 	    if (this_3dplot->plot_style == PM3DSURFACE)
 		return;
 	    if (this_3dplot->plot_style == IMAGE)
 		return;
-	    if (this_3dplot->lp_properties.use_palette) {
-	        int type = this_3dplot->lp_properties.pm3d_color.type;
-		if (type == TC_LT || type == TC_LINESTYLE || type == TC_RGB)
-		    /* don't return yet */
-		    ;
-		else
-		    /* TC_DEFAULT: splot x with line|lp|dot palette */
-		    return;
-	    }
+
+	    type = this_3dplot->lp_properties.pm3d_color.type;
+	    if (type == TC_LT || type == TC_LINESTYLE || type == TC_RGB)
+		; /* don't return yet */
+	    else
+		/* TC_DEFAULT: splot x with line|lp|dot palette */
+		return;
+
 	    if (this_3dplot->labels &&
 		this_3dplot->labels->textcolor.type >= TC_CB)
 		return;
@@ -1156,8 +1280,9 @@ set_plot_with_palette(int plot_num, int plot_mode)
 	}
     }
 
-    /* Any label with 'textcolor palette'? */
 #define TC_USES_PALETTE(tctype) (tctype==TC_Z) || (tctype==TC_CB) || (tctype==TC_FRAC)
+
+    /* Any label with 'textcolor palette'? */
     for (; this_label != NULL; this_label = this_label->next) {
 	if (TC_USES_PALETTE(this_label->textcolor.type))
 	    return;
@@ -1171,6 +1296,13 @@ set_plot_with_palette(int plot_num, int plot_mode)
     if (plot_mode == MODE_SPLOT)
 	if (TC_USES_PALETTE(axis_array[FIRST_Z_AXIS].label.textcolor.type)) return;
     if (TC_USES_PALETTE(axis_array[COLOR_AXIS].label.textcolor.type)) return;
+#ifdef EAM_OBJECTS
+    for (this_object = first_object; this_object != NULL; this_object = this_object->next) {
+	if (TC_USES_PALETTE(this_object->lp_properties.pm3d_color.type))
+	    return;
+    }
+#endif
+    
 #undef TC_USES_PALETTE
 
     /* Palette with continuous colors is not used. */
@@ -1189,3 +1321,108 @@ is_plot_with_colorbox()
     return plot_has_palette && (color_box.where != SMCOLOR_BOX_NO);
 }
 
+/*
+ * Adjust current RGB color based on pm3d lighting model.
+ */
+int
+apply_lighting_model( struct coordinate *v0, struct coordinate *v1, 
+		struct coordinate *v2, struct coordinate *v3,
+		double gray )
+{
+    double normal[3];
+    double normal1[3];
+    double reflect[3];
+    double t;
+    double phi;
+    double psi;
+    int rgb;
+    rgb_color color;
+    double r, g, b, tmp_r, tmp_g, tmp_b;
+    double dot_prod, shade_fact, spec_fact;
+
+    if (color_from_rgbvar) {
+	rgb = gray;
+	r = (double)((rgb >> 16) & 0xFF) / 255.;
+	g = (double)((rgb >>  8) & 0xFF) / 255.;
+	b = (double)((rgb      ) & 0xFF) / 255.;
+    } else {
+	rgb1_from_gray(gray, &color);
+	r = color.r;
+	g = color.g;
+	b = color.b;	   
+    }
+
+    psi = -DEG2RAD*(surface_rot_z);
+    phi = -DEG2RAD*(surface_rot_x);
+
+    normal[0] = (v1->y-v0->y)*(v2->z-v0->z)*yscale3d*zscale3d
+	      - (v1->z-v0->z)*(v2->y-v0->y)*yscale3d*zscale3d;
+    normal[1] = (v1->z-v0->z)*(v2->x-v0->x)*xscale3d*zscale3d
+	      - (v1->x-v0->x)*(v2->z-v0->z)*xscale3d*zscale3d;
+    normal[2] = (v1->x-v0->x)*(v2->y-v0->y)*xscale3d*yscale3d
+	      - (v1->y-v0->y)*(v2->x-v0->x)*xscale3d*yscale3d ;
+
+    t = sqrt( normal[0]*normal[0] + normal[1]*normal[1] + normal[2]*normal[2] );
+
+    normal[0] /= t;    
+    normal[1] /= t;    
+    normal[2] /= t;    
+
+    /* Correct for the view angle so that the illumination is "fixed" with */
+    /* respect to the viewer rather than rotating with the surface.        */
+    if (pm3d_shade.fixed) {
+	normal1[0] =  cos(psi)*normal[0] -  sin(psi)*normal[1] + 0*normal[2];
+	normal1[1] =  sin(psi)*normal[0] +  cos(psi)*normal[1] + 0*normal[2];
+	normal1[2] =  0*normal[0] +                0*normal[1] + 1*normal[2];
+	
+	normal[0] =  1*normal1[0] +         0*normal1[1] +         0*normal1[2];
+	normal[1] =  0*normal1[0] +  cos(phi)*normal1[1] -  sin(phi)*normal1[2];
+	normal[2] =  0*normal1[0] +  sin(phi)*normal1[1] +  cos(phi)*normal1[2];
+    }
+
+    if (normal[2] < 0.0) {
+	normal[0] *= -1.0;
+	normal[1] *= -1.0;
+	normal[2] *= -1.0;
+    }
+
+    dot_prod = normal[0]*light[0] + normal[1]*light[1] + normal[2]*light[2];
+    shade_fact = (dot_prod < 0) ? -dot_prod : 0;
+
+    tmp_r = r*(pm3d_shade.ambient-pm3d_shade.strength+shade_fact*pm3d_shade.strength);
+    tmp_g = g*(pm3d_shade.ambient-pm3d_shade.strength+shade_fact*pm3d_shade.strength);
+    tmp_b = b*(pm3d_shade.ambient-pm3d_shade.strength+shade_fact*pm3d_shade.strength);
+
+    /* Specular highlighting */ 
+    if (pm3d_shade.spec > 0.0) {
+
+	reflect[0] = -light[0]+2*dot_prod*normal[0];
+	reflect[1] = -light[1]+2*dot_prod*normal[1];
+	reflect[2] = -light[2]+2*dot_prod*normal[2];
+	t = sqrt( reflect[0]*reflect[0] + reflect[1]*reflect[1] + reflect[2]*reflect[2] );
+	reflect[0] /= t;
+	reflect[1] /= t;
+	reflect[2] /= t;
+	
+	dot_prod = -reflect[2];
+	if (dot_prod < 0.0)
+	    dot_prod = 0;
+
+	/* old-style Phong equation; no need for bells or whistles */
+	spec_fact = pow(dot_prod, pm3d_shade.Phong);
+
+	tmp_r += pm3d_shade.spec*spec_fact;
+	tmp_g += pm3d_shade.spec*spec_fact;
+	tmp_b += pm3d_shade.spec*spec_fact;
+    }
+
+    tmp_r = clip_to_01(tmp_r);
+    tmp_g = clip_to_01(tmp_g);
+    tmp_b = clip_to_01(tmp_b);
+
+    rgb = ((unsigned char)((tmp_r)*255.) << 16)
+	+ ((unsigned char)((tmp_g)*255.) <<  8)
+	+ ((unsigned char)((tmp_b)*255.));
+
+    return rgb;
+}
