@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: show.c,v 1.326.2.20 2017/02/25 05:20:33 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: show.c,v 1.385.2.1 2017/06/10 18:05:14 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - show.c */
@@ -52,6 +52,7 @@ static char *RCSid() { return RCSid("$Id: show.c,v 1.326.2.20 2017/02/25 05:20:3
 #include "gp_time.h"
 #include "graphics.h"
 #include "hidden3d.h"
+#include "jitter.h"
 #include "misc.h"
 #include "gp_hist.h"
 #include "plot2d.h"
@@ -82,7 +83,6 @@ static void show_at __PROTO((void));
 static void disp_at __PROTO((struct at_type *, int));
 static void show_all __PROTO((void));
 static void show_autoscale __PROTO((void));
-static void show_bars __PROTO((void));
 static void show_border __PROTO((void));
 static void show_boxwidth __PROTO((void));
 static void show_boxplot __PROTO((void));
@@ -145,10 +145,11 @@ static void show_size __PROTO((void));
 static void show_origin __PROTO((void));
 static void show_term __PROTO((void));
 static void show_tics __PROTO((TBOOLEAN showx, TBOOLEAN showy, TBOOLEAN showz, TBOOLEAN showx2, TBOOLEAN showy2, TBOOLEAN showcb));
-static void show_mtics __PROTO((AXIS_INDEX));
+static void show_mtics __PROTO((struct axis *));
 static void show_timestamp __PROTO((void));
 static void show_range __PROTO((AXIS_INDEX axis));
 static void show_link __PROTO((void));
+static void show_nonlinear __PROTO((void));
 static void show_xyzlabel __PROTO((const char *name, const char *suffix, text_label * label));
 static void show_title __PROTO((void));
 static void show_axislabel __PROTO((AXIS_INDEX));
@@ -173,13 +174,13 @@ static void show_arrowstyle __PROTO((int tag));
 static void show_arrow __PROTO((int tag));
 
 static void show_ticdef __PROTO((AXIS_INDEX));
-       void show_position __PROTO((struct position * pos));
+static void show_ticdefp __PROTO((struct axis *));
+       void show_position __PROTO((struct position * pos, int ndim));
 static void show_functions __PROTO((void));
 
 static int var_show_all = 0;
 
 /* following code segments appear over and over again */
-#define SHOW_NUM_OR_TIME(x, axis) save_num_or_time_input(stderr, x, axis)
 #define SHOW_ALL_NL { if (!var_show_all) (void) putc('\n',stderr); }
 
 #define PROGRAM "G N U P L O T"
@@ -214,7 +215,7 @@ show_command()
 	show_autoscale();
 	break;
     case S_BARS:
-	show_bars();
+	save_bars(stderr);
 	break;
     case S_BIND:
 	while (!END_OF_COMMAND) c_token++;
@@ -266,7 +267,9 @@ show_command()
 	break;
     case S_ZEROAXIS:
 	show_zeroaxis(FIRST_X_AXIS);
+	show_zeroaxis(SECOND_X_AXIS);
 	show_zeroaxis(FIRST_Y_AXIS);
+	show_zeroaxis(SECOND_Y_AXIS);
 	show_zeroaxis(FIRST_Z_AXIS);
 	break;
     case S_XZEROAXIS:
@@ -326,6 +329,9 @@ show_command()
     case S_LINK:
 	show_link();
 	break;
+    case S_NONLINEAR:
+	show_nonlinear();
+	break;
     case S_KEY:
 	show_key();
 	break;
@@ -414,6 +420,9 @@ show_command()
     case S_ISOSAMPLES:
 	show_isosamples();
 	break;
+    case S_JITTER:
+	show_jitter();
+	break;
     case S_VIEW:
 	show_view();
 	break;
@@ -445,29 +454,37 @@ show_command()
     case S_TICS:
     case S_TICSLEVEL:
     case S_TICSCALE:
-    case S_XYPLANE:
 	show_tics(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE);
 	break;
     case S_MXTICS:
-	show_mtics(FIRST_X_AXIS);
+	show_mtics(&axis_array[FIRST_X_AXIS]);
 	break;
     case S_MYTICS:
-	show_mtics(FIRST_Y_AXIS);
+	show_mtics(&axis_array[FIRST_Y_AXIS]);
 	break;
     case S_MZTICS:
-	show_mtics(FIRST_Z_AXIS);
+	show_mtics(&axis_array[FIRST_Z_AXIS]);
 	break;
     case S_MCBTICS:
-	show_mtics(COLOR_AXIS);
+	show_mtics(&axis_array[COLOR_AXIS]);
 	break;
     case S_MX2TICS:
-	show_mtics(SECOND_X_AXIS);
+	show_mtics(&axis_array[SECOND_X_AXIS]);
 	break;
     case S_MY2TICS:
-	show_mtics(SECOND_Y_AXIS);
+	show_mtics(&axis_array[SECOND_Y_AXIS]);
 	break;
     case S_MRTICS:
-	show_mtics(POLAR_AXIS);
+	show_mtics(&R_AXIS);
+	break;
+    case S_MTTICS:
+	show_mtics(&THETA_AXIS);
+	break;
+    case S_XYPLANE:
+	if (xyplane.absolute)
+	    fprintf(stderr, "\txyplane intercepts z axis at %g\n", xyplane.z);
+	else
+	    fprintf(stderr, "\txyplane %g\n", xyplane.z);
 	break;
     case S_TIMESTAMP:
 	show_timestamp();
@@ -517,6 +534,9 @@ show_command()
     case S_CBLABEL:
 	show_axislabel(COLOR_AXIS);
 	break;
+    case S_RLABEL:
+	show_axislabel(POLAR_AXIS);
+	break;
     case S_X2LABEL:
 	show_axislabel(SECOND_X_AXIS);
 	break;
@@ -563,7 +583,7 @@ show_command()
 #endif
     case S_PLOT:
 	show_plot();
-#if defined(READLINE) || defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(USE_READLINE)
 	if (!END_OF_COMMAND) {
 	    if (almost_equals(c_token, "a$dd2history")) {
 		c_token++;
@@ -599,6 +619,9 @@ show_command()
     case S_RTICS:
 	show_ticdef(POLAR_AXIS);
 	break;
+    case S_TTICS:
+	show_ticdefp(&THETA_AXIS);
+	break;
     case S_X2TICS:
     case S_X2DTICS:
     case S_X2MTICS:
@@ -619,6 +642,13 @@ show_command()
 		(*term_options) ? term_options : "[none]");
 	break;
 
+    case S_THETA:
+	fprintf(stderr,"Theta increases %s with origin at %s of plot\n",
+		theta_direction > 0 ? "counterclockwise" : "clockwise",
+		theta_origin == 180 ? "left" : theta_origin ==  90 ? "top" :
+		theta_origin == -90 ? "bottom" : "right");
+	break;
+
     /* HBB 20010525: 'set commands' that don't have an
      * accompanying 'show' version, for no particular reason: */
     /* --- such case now, all implemented. */
@@ -736,7 +766,7 @@ show_all()
 
     show_version(stderr);
     show_autoscale();
-    show_bars();
+    save_bars(stderr);
     show_border();
     show_boxwidth();
     show_clip();
@@ -785,11 +815,11 @@ show_all()
     show_origin();
     show_term();
     show_tics(TRUE,TRUE,TRUE,TRUE,TRUE,TRUE);
-    show_mtics(FIRST_X_AXIS);
-    show_mtics(FIRST_Y_AXIS);
-    show_mtics(FIRST_Z_AXIS);
-    show_mtics(SECOND_X_AXIS);
-    show_mtics(SECOND_Y_AXIS);
+    show_mtics(&axis_array[FIRST_X_AXIS]);
+    show_mtics(&axis_array[FIRST_Y_AXIS]);
+    show_mtics(&axis_array[FIRST_Z_AXIS]);
+    show_mtics(&axis_array[SECOND_X_AXIS]);
+    show_mtics(&axis_array[SECOND_Y_AXIS]);
     show_xyzlabel("", "time", &timelabel);
     if (parametric || polar) {
 	if (!is_3d_plot)
@@ -804,6 +834,7 @@ show_all()
     show_range(SECOND_X_AXIS);
     show_range(SECOND_Y_AXIS);
     show_range(FIRST_Z_AXIS);
+    show_jitter();
     show_title();
     show_axislabel(FIRST_X_AXIS );
     show_axislabel(FIRST_Y_AXIS );
@@ -930,10 +961,6 @@ show_version(FILE *fp)
 #endif
 		"";
 
-	    const char *binary_files =
-		"+BINARY_DATA  "
-		"";
-
 	    const char *nocwdrc =
 #ifdef USE_CWDRC
 		"+"
@@ -945,10 +972,6 @@ show_version(FILE *fp)
 	    const char *x11 =
 #ifdef X11
 		"+X11  "
-		"+X11_POLYGON  "
-#ifdef USE_X11_MULTIBYTE
-		"+MULTIBYTE  "
-#endif
 #ifdef EXTERNAL_X11_WINDOW
 		"+X11_EXTERNAL "
 #endif
@@ -972,16 +995,9 @@ show_version(FILE *fp)
 		"";
 
 	    const char *plotoptions=
-		"+DATASTRINGS  "
-		"+HISTOGRAMS  "
 #ifdef EAM_OBJECTS
 		"+OBJECTS  "
 #endif
-		"+STRINGVARS  "
-		"+MACROS  "
-		"+THIN_SPLINES  "
-		"+IMAGE  "
-		"+USER_LINETYPES "
 #ifdef USE_STATS
 		"+STATS "
 #else
@@ -992,14 +1008,18 @@ show_version(FILE *fp)
 #endif
 	    "";
 
-	    sprintf(compile_options, "\
-%s%s\n%s%s\n\
-%s%s%s\n\
-%s%s%s%s\n%s\n",
-		    rdline, gnu_rdline, compatibility, binary_files,
+	    const char *unicodebuild =
+#if defined(_WIN32) && defined(UNICODE)
+		"+UNICODE  ";
+#else
+		"";
+#endif
+
+	    sprintf(compile_options, "    %s%s\n    %s%s%s\n    %s%s%s\n    %s%s%s%s\n",
+		    rdline, gnu_rdline, compatibility, unicodebuild, plotoptions,
 		    libcerf, libgd, linuxvga,
-		    nocwdrc, x11, use_mouse, hiddenline,
-		    plotoptions);
+		    nocwdrc, x11, use_mouse, hiddenline
+		    );
 	}
 
 	compile_options = gp_realloc(compile_options, strlen(compile_options)+1, "compile_options");
@@ -1061,8 +1081,8 @@ show_version(FILE *fp)
     if (almost_equals(c_token, "l$ong")) {
 
 	c_token++;
-	fprintf(stderr, "Compile options:\n%s", compile_options);
-	fprintf(stderr, "MAX_PARALLEL_AXES=%d\n\n", MAX_PARALLEL_AXES);
+	fprintf(stderr, "\nCompile options:\n%s", compile_options);
+	fprintf(stderr, "    MAX_PARALLEL_AXES=%d\n\n", MAX_PARALLEL_AXES);
 
 #ifdef X11
 	{
@@ -1086,19 +1106,19 @@ show_version(FILE *fp)
 	}
 
 	{
+#ifndef _WIN32
 	    char *helpfile = NULL;
 
-#ifndef WIN32
 	    if ((helpfile = getenv("GNUHELP")) == NULL)
 		helpfile = HELPFILE;
+	    fprintf(stderr, "HELPFILE           = \"%s\"\n", helpfile);
 #else
-	    helpfile = winhelpname;
+	    fprintf(stderr, "HELPFILE           = \"" TCHARFMT "\"\n", winhelpname);
 #endif
-	fprintf(stderr, "HELPFILE           = \"%s\"\n", helpfile);
 	}
 
-#if defined(WIN32) && !defined(WGP_CONSOLE)
-	fprintf(stderr, "MENUNAME           = \"%s\"\n", szMenuName);
+#if defined(_WIN32) && !defined(WGP_CONSOLE)
+	fprintf(stderr, "MENUNAME           = \"" TCHARFMT "\"\n", szMenuName);
 #endif
 
 #ifdef HAVE_LIBCACA
@@ -1129,12 +1149,9 @@ show_autoscale()
 
     fputs("\tautoscaling is ", stderr);
     if (parametric) {
-	if (is_3d_plot) {
-	    SHOW_AUTOSCALE(T_AXIS);
-	} else {
-	    SHOW_AUTOSCALE(U_AXIS);
-	    SHOW_AUTOSCALE(V_AXIS);
-	}
+	SHOW_AUTOSCALE(T_AXIS);
+	SHOW_AUTOSCALE(U_AXIS);
+	SHOW_AUTOSCALE(V_AXIS);
     }
 
     if (polar) {
@@ -1154,21 +1171,6 @@ show_autoscale()
 }
 
 
-/* process 'show bars' command */
-static void
-show_bars()
-{
-    SHOW_ALL_NL;
-
-    /* I really like this: "terrorbars" ;-) */
-    if (bar_size > 0.0)
-	fprintf(stderr, "\terrorbars are plotted in %s with bars of size %f\n",
-		(bar_layer == LAYER_BACK) ? "back" : "front", bar_size);
-    else
-	fputs("\terrors are plotted without bars\n", stderr);
-}
-
-
 /* process 'show border' command */
 static void
 show_border()
@@ -1178,8 +1180,8 @@ show_border()
     if (!draw_border)
 	fprintf(stderr, "\tborder is not drawn\n");
     else {
-	fprintf(stderr, "\tborder %d is drawn in %s layer with\n\t ",
-	    draw_border, 
+	fprintf(stderr, "\tborder %d (0x%X) is drawn in %s layer with\n\t ",
+	    draw_border, draw_border, 
 	    border_layer == LAYER_BEHIND ? "behind" : border_layer == LAYER_BACK ? "back" : "front");
 	save_linetype(stderr, &border_lp, FALSE);
 	fputc('\n',stderr);
@@ -1275,15 +1277,11 @@ show_clip()
 
     fprintf(stderr, "\tpoint clip is %s\n", (clip_points) ? "ON" : "OFF");
 
-    if (clip_lines1)
-	fputs("\tdrawing and clipping lines between inrange and outrange points\n", stderr);
-    else
-	fputs("\tnot drawing lines between inrange and outrange points\n", stderr);
+    fprintf(stderr, "\t%s lines with one end out of range (clip one)\n",
+	clip_lines1 ? "clipping" : "not drawing");
 
-    if (clip_lines2)
-	fputs("\tdrawing and clipping lines between two outrange points\n", stderr);
-    else
-	fputs("\tnot drawing lines between two outrange points\n", stderr);
+    fprintf(stderr, "\t%s lines with both ends out of range (clip two)\n",
+	clip_lines2 ? "clipping" : "not drawing");
 }
 
 
@@ -1384,13 +1382,13 @@ show_dgrid3d()
     SHOW_ALL_NL;
 
     if (dgrid3d)
-      if( dgrid3d_mode == DGRID3D_QNORM ) {
+      if (dgrid3d_mode == DGRID3D_QNORM) {
 	fprintf(stderr,
 		"\tdata grid3d is enabled for mesh of size %dx%d, norm=%d\n",
 		dgrid3d_row_fineness,
 		dgrid3d_col_fineness,
 		dgrid3d_norm_value );
-      } else if( dgrid3d_mode == DGRID3D_SPLINES ){
+      } else if (dgrid3d_mode == DGRID3D_SPLINES) {
 	fprintf(stderr,
 		"\tdata grid3d is enabled for mesh of size %dx%d, splines\n",
 		dgrid3d_row_fineness,
@@ -1457,20 +1455,13 @@ show_format()
     SHOW_ALL_NL;
 
     fprintf(stderr, "\ttic format is:\n");
-#define SHOW_FORMAT(_axis)						\
-    fprintf(stderr, "\t  %s-axis: \"%s\"%s\n", axis_name(_axis),	\
-	    conv_text(axis_array[_axis].formatstring),			\
-	    axis_array[_axis].tictype == DT_DMS ? " geographic" :	\
-	    axis_array[_axis].tictype == DT_TIMEDATE ? " time" :	\
-	    "");
-    SHOW_FORMAT(FIRST_X_AXIS );
-    SHOW_FORMAT(FIRST_Y_AXIS );
-    SHOW_FORMAT(SECOND_X_AXIS);
-    SHOW_FORMAT(SECOND_Y_AXIS);
-    SHOW_FORMAT(FIRST_Z_AXIS );
-    SHOW_FORMAT(COLOR_AXIS);
-    SHOW_FORMAT(POLAR_AXIS);
-#undef SHOW_FORMAT
+    save_axis_format(stderr, FIRST_X_AXIS );
+    save_axis_format(stderr, FIRST_Y_AXIS );
+    save_axis_format(stderr, SECOND_X_AXIS);
+    save_axis_format(stderr, SECOND_Y_AXIS);
+    save_axis_format(stderr, FIRST_Z_AXIS );
+    save_axis_format(stderr, COLOR_AXIS);
+    save_axis_format(stderr, POLAR_AXIS);
 }
 
 
@@ -1593,7 +1584,7 @@ show_style_circle()
 {
     SHOW_ALL_NL;
     fprintf(stderr, "\tCircle style has default radius ");
-    show_position(&default_circle.o.circle.extent);
+    show_position(&default_circle.o.circle.extent, 1);
     fprintf(stderr, " [%s]", default_circle.o.circle.wedge ? "wedge" : "nowedge");
     fputs("\n", stderr);
 }
@@ -1603,7 +1594,7 @@ show_style_ellipse()
 {
     SHOW_ALL_NL;
     fprintf(stderr, "\tEllipse style has default size ");
-    show_position(&default_ellipse.o.ellipse.extent);
+    show_position(&default_ellipse.o.ellipse.extent, 2);
     fprintf(stderr, ", default angle is %.1f degrees", default_ellipse.o.ellipse.orientation);
 
     switch (default_ellipse.o.ellipse.type) {
@@ -1700,12 +1691,13 @@ static void
 show_paxis()
 {
     int p = int_expression();
-    if (p <=0 || p > MAX_PARALLEL_AXES)
-	int_error(c_token, "expecting parallel axis number 1 - %d",MAX_PARALLEL_AXES);
+    if (p <=0 || p > num_parallel_axes)
+	int_error(c_token, "no such parallel axis is active");
+    fputs("\n\t", stderr);
     if (equals(c_token, "range"))
-	show_range(PARALLEL_AXES+p-1);
+	save_prange(stderr, &parallel_axis[p-1]);
     else if (almost_equals(c_token, "tic$s"))
-	show_ticdef(PARALLEL_AXES+p-1);
+	show_ticdefp(&parallel_axis[p-1]);
     c_token++;
 }
 
@@ -1721,12 +1713,6 @@ show_zeroaxis(AXIS_INDEX axis)
 	fputc('\n',stderr);
     } else
 	fprintf(stderr, "\t%szeroaxis is OFF\n", axis_name(axis));
-
-    if ((axis / SECOND_AXES) == 0) {
-	/* this is a 'first' axis. To output secondary axis, call self
-	 * recursively: */
-	show_zeroaxis(axis + SECOND_AXES);
-    }
 }
 
 /* Show label number <tag> (0 means show all) */
@@ -1743,7 +1729,7 @@ show_label(int tag)
 	    fprintf(stderr, "\tlabel %d \"%s\" at ",
 		    this_label->tag,
 		    (this_label->text==NULL) ? "" : conv_text(this_label->text));
-	    show_position(&this_label->place);
+	    show_position(&this_label->place, 3);
 	    if (this_label->hypertext)
 		fprintf(stderr, " hypertext");
 	    switch (this_label->pos) {
@@ -1777,7 +1763,7 @@ show_label(int tag)
 		fprintf(stderr, " point with color of");
 		save_linetype(stderr, &(this_label->lp_properties), TRUE);
 		fprintf(stderr, " offset ");
-		show_position(&this_label->offset);
+		show_position(&this_label->offset, 3);
 	    }
 
 #ifdef EAM_BOXED_TEXT
@@ -1815,16 +1801,16 @@ show_arrow(int tag)
 		    this_arrow->arrow_properties.layer ? "front" : "back");
 	    save_linetype(stderr, &(this_arrow->arrow_properties.lp_properties), FALSE);
 	    fprintf(stderr, "\n\t  from ");
-	    show_position(&this_arrow->start);
+	    show_position(&this_arrow->start, 3);
 	    if (this_arrow->type == arrow_end_absolute) {
 		fputs(" to ", stderr);
-		show_position(&this_arrow->end);
+		show_position(&this_arrow->end, 3);
 	    } else if (this_arrow->type == arrow_end_relative) {
 		fputs(" rto ", stderr);
-		show_position(&this_arrow->end);
+		show_position(&this_arrow->end, 3);
 	    } else { /* arrow_end_oriented */
 		fputs(" length ", stderr);
-		show_position(&this_arrow->end);
+		show_position(&this_arrow->end, 1);
 		fprintf(stderr," angle %g deg",this_arrow->angle);
 	    }
 	    if (this_arrow->arrow_properties.head_length > 0) {
@@ -1901,7 +1887,7 @@ show_key()
 	    fputs(" horizontal", stderr);
 	}
 	if (key->region == GPKEY_AUTO_INTERIOR_LRTBC)
-	    fputs(" inside", stderr);
+	    fputs(key->fixed ? " fixed" : " inside", stderr);
 	else if (key->region == GPKEY_AUTO_EXTERIOR_LRTBC)
 	    fputs(" outside", stderr);
 	else {
@@ -1925,7 +1911,7 @@ show_key()
     }
     case GPKEY_USER_PLACEMENT:
 	fputs("\tkey is at ", stderr);
-	show_position(&key->user_pos);
+	show_position(&key->user_pos, 2);
 	putc('\n', stderr);
 	break;
     }
@@ -1984,10 +1970,10 @@ show_key()
 
 
 void
-show_position(struct position *pos)
+show_position(struct position *pos, int ndim)
 {
     fprintf(stderr,"(");
-    save_position(stderr, pos, FALSE);
+    save_position(stderr, pos, ndim, FALSE);
     fprintf(stderr,")");
 }
 
@@ -2291,7 +2277,7 @@ show_palette_gradient()
 	return;
     }
 
-    for( i=0; i<sm_palette.gradient_num; ++i ) {
+    for (i=0; i<sm_palette.gradient_num; i++) {
         gray = sm_palette.gradient[i].pos;
         r = sm_palette.gradient[i].col.r;
         g = sm_palette.gradient[i].col.g;
@@ -2341,6 +2327,7 @@ show_palette()
 	    sm_palette.colorMode == SMPAL_COLOR_MODE_GRAY ? "GRAY" : "COLOR");
 
 	switch( sm_palette.colorMode ) {
+	  default:
 	  case SMPAL_COLOR_MODE_GRAY: break;
 	  case SMPAL_COLOR_MODE_RGB:
 	    fprintf(stderr,"\trgb color mapping by rgbformulae are %i,%i,%i\n",
@@ -2367,9 +2354,6 @@ show_palette()
 			sm_palette.cubehelix_start, sm_palette.cubehelix_cycles,
 			sm_palette.cubehelix_saturation);
 	    break;
-	  default:
-	    fprintf( stderr, "%s:%d oops: Unknown color mode '%c'.\n",
-		     __FILE__, __LINE__, (char)(sm_palette.colorMode) );
 	}
 	fprintf(stderr,"\tfigure is %s\n",
 	    sm_palette.positive == SMPAL_POSITIVE ? "POSITIVE" : "NEGATIVE");
@@ -2384,14 +2368,12 @@ show_palette()
 	fputs(" color positions for discrete palette terminals\n", stderr);
 	fputs( "\tColor-Model: ", stderr );
 	switch( sm_palette.cmodel ) {
+	default:
 	case C_MODEL_RGB: fputs( "RGB\n", stderr ); break;
 	case C_MODEL_HSV: fputs( "HSV\n", stderr ); break;
 	case C_MODEL_CMY: fputs( "CMY\n", stderr ); break;
 	case C_MODEL_YIQ: fputs( "YIQ\n", stderr ); break;
 	case C_MODEL_XYZ: fputs( "XYZ\n", stderr ); break;
-	default:
-	  fprintf( stderr, "%s:%d ooops: Unknown color mode '%c'.\n",
-		   __FILE__, __LINE__, (char)(sm_palette.cmodel) );
 	}
 	fprintf(stderr,"\tgamma is %.4g\n", sm_palette.gamma );
 	return;
@@ -2454,9 +2436,9 @@ show_colorbox()
 	    break;
 	case SMCOLOR_BOX_USER:
 	    fputs("at USER origin: ", stderr);
-	    show_position(&color_box.origin);
+	    show_position(&color_box.origin, 2);
 	    fputs("\n\t          size: ", stderr);
-	    show_position(&color_box.size);
+	    show_position(&color_box.size, 2);
 	    fputs("\n", stderr);
 	    break;
 	default: /* should *never* happen */
@@ -2513,7 +2495,10 @@ show_pm3d()
 	save_linetype(stderr, &(pm3d.border), FALSE);
 	fprintf(stderr,"\n");
     }
-
+    if (pm3d_shade.strength > 0) {
+	fprintf(stderr,"\tlighting primary component %g specular component %g\n",
+		pm3d_shade.strength, pm3d_shade.spec);
+    }
     fprintf(stderr,"\tsteps for bilinear interpolation: %d,%d\n",
 	 pm3d.interp_i, pm3d.interp_j);
     fprintf(stderr,"\tquadrangle color according to ");
@@ -2659,7 +2644,7 @@ show_fit()
     }
 
     v = get_udv_by_name((char *)FITLIMIT);
-    d = ((v != NULL) && (!v->udv_undef)) ? real(&(v->udv_value)) : -1.0;
+    d = ((v != NULL) && (v->udv_value.type != NOTDEFINED)) ? real(&(v->udv_value)) : -1.0;
     fprintf(stderr, "\tfits will be considered to have converged if  delta chisq < chisq * %g",
 	((d > 0.) && (d < 1.)) ? d : DEF_FIT_LIMIT);
     if (epsilon_abs > 0.)
@@ -2667,19 +2652,19 @@ show_fit()
     fprintf(stderr, "\n");
 
     v = get_udv_by_name((char *)FITMAXITER);
-    if ((v != NULL) && (!v->udv_undef) && (real_int(&(v->udv_value)) > 0))
+    if ((v != NULL) && (v->udv_value.type != NOTDEFINED) && (real_int(&(v->udv_value)) > 0))
 	fprintf(stderr, "\tfit will stop after a maximum of %i iterations\n",
 	        real_int(&(v->udv_value)));
     else
 	fprintf(stderr, "\tfit has no limit in the number of iterations\n");
 
     v = get_udv_by_name((char *)FITSTARTLAMBDA);
-    d = ((v != NULL) && (!v->udv_undef)) ? real(&(v->udv_value)) : -1.0;
+    d = ((v != NULL) && (v->udv_value.type != NOTDEFINED)) ? real(&(v->udv_value)) : -1.0;
     if (d > 0.)
 	fprintf(stderr, "\tfit will start with lambda = %g\n", d);
 
     v = get_udv_by_name((char *)FITLAMBDAFACTOR);
-    d = ((v != NULL) && (!v->udv_undef)) ? real(&(v->udv_value)) : -1.0;
+    d = ((v != NULL) && (v->udv_value.type != NOTDEFINED)) ? real(&(v->udv_value)) : -1.0;
     if (d > 0.)
 	fprintf(stderr, "\tfit will change lambda by a factor of %g\n", d);
 
@@ -2689,7 +2674,7 @@ show_fit()
 	fprintf(stderr, "\tfit will default to `unitweights` if no `error`keyword is given on the command line.\n");
     fprintf(stderr, "\tfit can run the following command when interrupted:\n\t\t'%s'\n", getfitscript());
     v = get_udv_by_name("GPVAL_LAST_FIT");
-    if (v != NULL && !v->udv_undef)
+    if (v != NULL && v->udv_value.type != NOTDEFINED)
 	fprintf(stderr, "\tlast fit command was: %s\n", v->udv_value.v.string_val);
 }
 
@@ -2752,6 +2737,7 @@ show_view()
     fprintf(stderr,"\t\t%s axes are %s\n",
 		aspect_ratio_3D == 2 ? "x/y" : aspect_ratio_3D == 3 ? "x/y/z" : "",
 		aspect_ratio_3D >= 2 ? "on the same scale" : "independently scaled");
+    fprintf(stderr, "\t\t azimuth %g\n", azimuth);
 }
 
 
@@ -2797,11 +2783,8 @@ show_histogram()
 static void
 show_textbox()
 {
-	fprintf(stderr, "\ttextboxes are %s ",
-		textbox_opts.opaque ? "opaque" : "transparent");
-	fprintf(stderr, "with margins %4.1f, %4.1f  and %s border\n",
-		textbox_opts.xmargin, textbox_opts.ymargin,
-		textbox_opts.noborder ? "no" : "");
+    fprintf(stderr, "\ttextbox style");
+    save_style_textbox(stderr);
 }
 #endif
 
@@ -2868,11 +2851,6 @@ show_tics(
     int i;
     SHOW_ALL_NL;
 
-    if (xyplane.absolute)
-	fprintf(stderr, "\txyplane intercepts z axis at %g\n", xyplane.z);
-    else
-	fprintf(stderr, "\txyplane ticslevel is %g\n", xyplane.z);
-
     fprintf(stderr, "\ttics are in %s of plot\n", (grid_tics_in_front) ? "front" : "back");
 
     if (showx)
@@ -2898,24 +2876,27 @@ show_tics(
 
 /* process 'show m[xyzx2y2cb]tics' commands */
 static void
-show_mtics(AXIS_INDEX axis)
+show_mtics(struct axis *axis)
 {
-    switch (axis_array[axis].minitics) {
+    char *name = axis_name(axis->index);
+
+    switch (axis->minitics) {
     case MINI_OFF:
-	fprintf(stderr, "\tminor %stics are off\n", axis_name(axis));
+	fprintf(stderr, "\tminor %stics are off\n", name);
 	break;
     case MINI_DEFAULT:
 	fprintf(stderr, "\
 \tminor %stics are off for linear scales\n\
-\tminor %stics are computed automatically for log scales\n", axis_name(axis), axis_name(axis));
+\tminor %stics are computed automatically for log scales\n", 
+	name, name);
 	break;
     case MINI_AUTO:
-	fprintf(stderr, "\tminor %stics are computed automatically\n", axis_name(axis));
+	fprintf(stderr, "\tminor %stics are computed automatically\n", name);
 	break;
     case MINI_USER:
 	fprintf(stderr, "\
 \tminor %stics are drawn with %d subintervals between major xtic marks\n",
-		axis_name(axis), (int) axis_array[axis].mtic_freq);
+		name, (int) axis->mtic_freq);
 	break;
     default:
 	int_error(NO_CARET, "Unknown minitic type in show_mtics()");
@@ -2931,10 +2912,6 @@ show_timestamp()
     show_xyzlabel("", "time", &timelabel);
     fprintf(stderr, "\twritten in %s corner\n",
 	    (timelabel_bottom ? "bottom" : "top"));
-    if (timelabel_rotate)
-	fputs("\trotated if the terminal allows it\n\t", stderr);
-    else
-	fputs("\tnot rotated\n\t", stderr);
 }
 
 
@@ -2946,7 +2923,7 @@ show_range(AXIS_INDEX axis)
     if (axis_array[axis].datatype == DT_TIMEDATE)
 	fprintf(stderr, "\tset %sdata time\n", axis_name(axis));
     fprintf(stderr,"\t");
-    save_range(stderr, axis);
+    save_prange(stderr, axis_array + axis);
 }
 
 
@@ -2957,7 +2934,7 @@ show_xyzlabel(const char *name, const char *suffix, text_label *label)
     if (label) {
 	fprintf(stderr, "\t%s%s is \"%s\", offset at ", name, suffix,
 	    label->text ? conv_text(label->text) : "");
-	show_position(&label->offset);
+	show_position(&label->offset, 3);
     } else
 	return;
 
@@ -3023,21 +3000,28 @@ static void
 show_link()
 {
     if (END_OF_COMMAND || almost_equals(c_token,"x$2"))
-	if (axis_array[SECOND_X_AXIS].linked_to_primary)
-	    save_range(stderr, SECOND_X_AXIS);
+	    save_link(stderr, axis_array + SECOND_X_AXIS);
     if (END_OF_COMMAND || almost_equals(c_token,"y$2"))
-	if (axis_array[SECOND_Y_AXIS].linked_to_primary)
-	    save_range(stderr, SECOND_Y_AXIS);
+	    save_link(stderr, axis_array + SECOND_Y_AXIS);
     if (!END_OF_COMMAND)
 	c_token++;
 }
 
+/* process 'show link' command */
+static void
+show_nonlinear()
+{
+    int axis;
+    for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++)
+	save_nonlinear(stderr, &axis_array[axis]);
+}
+
 /* process 'show locale' command */
 static void
 show_locale()
 {
     SHOW_ALL_NL;
-    locale_handler(ACTION_SHOW,NULL);
+    dump_locale();
 }
 
 
@@ -3046,7 +3030,7 @@ static void
 show_loadpath()
 {
     SHOW_ALL_NL;
-    loadpath_handler(ACTION_SHOW,NULL);
+    dump_loadpath();
 }
 
 
@@ -3055,7 +3039,7 @@ static void
 show_fontpath()
 {
     SHOW_ALL_NL;
-    fontpath_handler(ACTION_SHOW,NULL);
+    dump_fontpath();
 }
 
 
@@ -3211,7 +3195,7 @@ show_variables()
 	    udv = udv->next_udv;
 	    continue;
 	}
-	if (udv->udv_undef) {
+	if (udv->udv_value.type == NOTDEFINED) {
 	    FPRINTF((stderr, "\t%-*s is undefined\n", len, udv->udv_name));
 	} else {
 	    fprintf(stderr, "\t%-*s ", len, udv->udv_name);
@@ -3329,40 +3313,40 @@ show_arrowstyle(int tag)
 
 /* called by show_tics */
 static void
-show_ticdef(AXIS_INDEX axis)
+show_ticdefp(struct axis *this_axis)
 {
     struct ticmark *t;
 
-    const char *ticfmt = conv_text(axis_array[axis].formatstring);
+    const char *ticfmt = conv_text(this_axis->formatstring);
 
     fprintf(stderr, "\t%s-axis tics are %s, \
 \tmajor ticscale is %g and minor ticscale is %g\n",
-	    axis_name(axis),
-	    (axis_array[axis].tic_in ? "IN" : "OUT"),
-	    axis_array[axis].ticscale, axis_array[axis].miniticscale);
+	    axis_name(this_axis->index),
+	    (this_axis->tic_in ? "IN" : "OUT"),
+	    this_axis->ticscale, this_axis->miniticscale);
 
-    fprintf(stderr, "\t%s-axis tics:\t", axis_name(axis));
-    switch (axis_array[axis].ticmode & TICS_MASK) {
+    fprintf(stderr, "\t%s-axis tics:\t", axis_name(this_axis->index));
+    switch (this_axis->ticmode & TICS_MASK) {
     case NO_TICS:
 	fputs("OFF\n", stderr);
 	return;
     case TICS_ON_AXIS:
 	fputs("on axis", stderr);
-	if (axis_array[axis].ticmode & TICS_MIRROR)
-	    fprintf(stderr, " and mirrored %s", (axis_array[axis].tic_in ? "OUT" : "IN"));
+	if (this_axis->ticmode & TICS_MIRROR)
+	    fprintf(stderr, " and mirrored %s", (this_axis->tic_in ? "OUT" : "IN"));
 	break;
     case TICS_ON_BORDER:
 	fputs("on border", stderr);
-	if (axis_array[axis].ticmode & TICS_MIRROR)
+	if (this_axis->ticmode & TICS_MIRROR)
 	    fputs(" and mirrored on opposite border", stderr);
 	break;
     }
 
-    if (axis_array[axis].ticdef.rangelimited)
+    if (this_axis->ticdef.rangelimited)
 	fprintf(stderr, "\n\t  tics are limited to data range");
     fputs("\n\t  labels are ", stderr);
-    if (axis_array[axis].manual_justify) {
-    	switch (axis_array[axis].label.pos) {
+    if (this_axis->manual_justify) {
+    	switch (this_axis->tic_pos) {
     	case LEFT:{
 		fputs("left justified, ", stderr);
 		break;
@@ -3380,22 +3364,22 @@ show_ticdef(AXIS_INDEX axis)
         fputs("justified automatically, ", stderr);
     fprintf(stderr, "format \"%s\"", ticfmt);
     fprintf(stderr, "%s", 
-	axis_array[axis].tictype == DT_DMS ? " geographic" :
-	axis_array[axis].tictype == DT_TIMEDATE ? " timedate" :
+	this_axis->tictype == DT_DMS ? " geographic" :
+	this_axis->tictype == DT_TIMEDATE ? " timedate" :
 	"");
-    if (axis_array[axis].ticdef.enhanced == FALSE)
+    if (this_axis->ticdef.enhanced == FALSE)
 	fprintf(stderr,"  noenhanced");
-    if (axis_array[axis].tic_rotate) {
+    if (this_axis->tic_rotate) {
 	fprintf(stderr," rotated");
-	fprintf(stderr," by %d",axis_array[axis].tic_rotate);
+	fprintf(stderr," by %d",this_axis->tic_rotate);
 	fputs(" in 2D mode, terminal permitting,\n\t", stderr);
     } else
 	fputs(" and are not rotated,\n\t", stderr);
     fputs("    offset ",stderr);
-    show_position(&axis_array[axis].ticdef.offset);
+    show_position(&this_axis->ticdef.offset, 3);
     fputs("\n\t",stderr);
 
-    switch (axis_array[axis].ticdef.type) {
+    switch (this_axis->ticdef.type) {
     case TIC_COMPUTED:{
 	    fputs("  intervals computed automatically\n", stderr);
 	    break;
@@ -3410,15 +3394,15 @@ show_ticdef(AXIS_INDEX axis)
 	}
     case TIC_SERIES:{
 	    fputs("  series", stderr);
-	    if (axis_array[axis].ticdef.def.series.start != -VERYLARGE) {
+	    if (this_axis->ticdef.def.series.start != -VERYLARGE) {
 		fputs(" from ", stderr);
-		SHOW_NUM_OR_TIME(axis_array[axis].ticdef.def.series.start, axis);
+		save_num_or_time_input(stderr, this_axis->ticdef.def.series.start, this_axis);
 	    }
-	    fprintf(stderr, " by %g%s", axis_array[axis].ticdef.def.series.incr,
-		    axis_array[axis].datatype == DT_TIMEDATE ? " secs" : "");
-	    if (axis_array[axis].ticdef.def.series.end != VERYLARGE) {
+	    fprintf(stderr, " by %g%s", this_axis->ticdef.def.series.incr,
+		    this_axis->datatype == DT_TIMEDATE ? " secs" : "");
+	    if (this_axis->ticdef.def.series.end != VERYLARGE) {
 		fputs(" until ", stderr);
-		SHOW_NUM_OR_TIME(axis_array[axis].ticdef.def.series.end, axis);
+		save_num_or_time_input(stderr, this_axis->ticdef.def.series.end, this_axis);
 	    }
 	    putc('\n', stderr);
 	    break;
@@ -3433,12 +3417,12 @@ show_ticdef(AXIS_INDEX axis)
 	}
     }
 
-    if (axis_array[axis].ticdef.def.user) {
+    if (this_axis->ticdef.def.user) {
 	fputs("\t  explicit list (", stderr);
-	for (t = axis_array[axis].ticdef.def.user; t != NULL; t = t->next) {
+	for (t = this_axis->ticdef.def.user; t != NULL; t = t->next) {
 	    if (t->label)
 		fprintf(stderr, "\"%s\" ", conv_text(t->label));
-	    SHOW_NUM_OR_TIME(t->position, axis);
+	    save_num_or_time_input(stderr, t->position, this_axis);
 	    if (t->level)
 		fprintf(stderr," %d",t->level);
 	    if (t->next)
@@ -3447,17 +3431,24 @@ show_ticdef(AXIS_INDEX axis)
 	fputs(")\n", stderr);
     }
 
-    if (axis_array[axis].ticdef.textcolor.type != TC_DEFAULT) {
+    if (this_axis->ticdef.textcolor.type != TC_DEFAULT) {
         fputs("\t ", stderr);
-	save_textcolor(stderr, &axis_array[axis].ticdef.textcolor);
+	save_textcolor(stderr, &this_axis->ticdef.textcolor);
         fputs("\n", stderr);
     }
 
-    if (axis_array[axis].ticdef.font && *axis_array[axis].ticdef.font) {
-        fprintf(stderr,"\t  font \"%s\"\n", axis_array[axis].ticdef.font);
+    if (this_axis->ticdef.font && *this_axis->ticdef.font) {
+        fprintf(stderr,"\t  font \"%s\"\n", this_axis->ticdef.font);
     }
 }
 
+/* called by show_tics */
+static void
+show_ticdef(AXIS_INDEX axis)
+{
+    show_ticdefp(&axis_array[axis]);
+}
+
 /* Display a value in human-readable form. */
 void
 disp_value(FILE *fp, struct value *val, TBOOLEAN need_quotes)
