@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: tabulate.c,v 1.13.2.2 2012/09/05 23:35:46 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: tabulate.c,v 1.26.2.1 2017/07/20 18:32:13 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - tabulate.c */
@@ -41,11 +41,15 @@ static char *RCSid() { return RCSid("$Id: tabulate.c,v 1.13.2.2 2012/09/05 23:35
  * These routines used to masquerade as a terminal type "set term table",
  * but since version 4.2 they are controlled by "set table <foo>" independent
  * of terminal settings.
+ *
+ * BM - April 2014
+ * Support output to an in-memory named datablock.
  */
 
 #include "alloc.h"
 #include "axis.h"
 #include "datafile.h"
+#include "datablock.h"
 #include "gp_time.h"
 #include "graphics.h"
 #include "graph3d.h"
@@ -53,37 +57,64 @@ static char *RCSid() { return RCSid("$Id: tabulate.c,v 1.13.2.2 2012/09/05 23:35
 #include "plot3d.h"
 #include "tabulate.h"
 
+
+/* File or datablock for output during 'set table' mode */
+FILE *table_outfile = NULL;
+udvt_entry *table_var = NULL;
+TBOOLEAN table_mode = FALSE;
+char *table_sep = NULL;
+
 static char *expand_newline __PROTO((const char *in));
+static TBOOLEAN imploded __PROTO((curve_points *this_plot));
 
 static FILE *outfile;
 
-/* This routine got longer than is reasonable for a macro */
-#define OUTPUT_NUMBER(x,y) output_number(x,y,buffer)
-#define BUFFERSIZE 150
+#define OUTPUT_NUMBER(x,y) { output_number(x, y, buffer); len = strappend(&line, &size, len, buffer); }
+
+#define BUFFERSIZE 128
+
 
 static void
 output_number(double coord, int axis, char *buffer) {
+    if (isnan(coord)) {
+	sprintf(buffer, " NaN");
+
     /* treat timedata and "%s" output format as a special case:
      * return a number.
      * "%s" in combination with any other character is treated
      * like a normal time format specifier and handled in time.c
      */
-    if (axis_array[axis].datatype == DT_TIMEDATE &&
+    } else if (axis_array[axis].tictype == DT_TIMEDATE &&
 	strcmp(axis_array[axis].formatstring, "%s") == 0) {
 	gprintf(buffer, BUFFERSIZE, "%.0f", 1.0, coord);
-    } else
-    if (axis_array[axis].datatype == DT_TIMEDATE) {
+    } else if (axis_array[axis].tictype == DT_TIMEDATE) {
 	buffer[0] = '"';
-	gstrftime(buffer+1, BUFFERSIZE-1, axis_array[axis].formatstring, coord);
+	if (!strcmp(axis_array[axis].formatstring,DEF_FORMAT))
+	    gstrftime(buffer+1, BUFFERSIZE-1, timefmt, coord);
+	else
+	    gstrftime(buffer+1, BUFFERSIZE-1, axis_array[axis].formatstring, coord);
 	while (strchr(buffer,'\n')) {*(strchr(buffer,'\n')) = ' ';}
 	strcat(buffer,"\"");
+#if !defined(NONLINEAR_AXES) || (NONLINEAR_AXES == 0)
     } else if (axis_array[axis].log) {
 	double x = pow(axis_array[axis].base, coord);
 	gprintf(buffer, BUFFERSIZE, axis_array[axis].formatstring, 1.0, x);
+#endif
     } else
 	gprintf(buffer, BUFFERSIZE, axis_array[axis].formatstring, 1.0, coord);
-    fputs(buffer, outfile);
-    fputc(' ', outfile);
+    strcat(buffer, " ");
+}
+
+
+static void
+print_line(const char *str)
+{
+    if (table_var == NULL) {
+	fputs(str, outfile);
+	fputc('\n', outfile);
+    } else {
+	append_to_datablock(&table_var->udv_value, strdup(str));
+    }
 }
 
 
@@ -91,51 +122,62 @@ void
 print_table(struct curve_points *current_plot, int plot_num)
 {
     int i, curve;
-    char *buffer = gp_alloc(BUFFERSIZE, "print_table: output buffer");
+    char *buffer = (char *) gp_alloc(BUFFERSIZE, "print_table: output buffer");
+    size_t size = 2*BUFFERSIZE;
+    char *line = (char *) gp_alloc(size, "print_table: line buffer");
+    size_t len = 0;
+
     outfile = (table_outfile) ? table_outfile : gpoutfile;
 
     for (curve = 0; curve < plot_num;
 	 curve++, current_plot = current_plot->next) {
-	struct coordinate GPHUGE *point = NULL;
+	struct coordinate *point = NULL;
 
-	/* two blank lines between tabulated plots by prepending \n here */
-	fprintf(outfile, "\n# Curve %d of %d, %d points",
+	/* "with table" already wrote the output */
+	if (current_plot->plot_style == TABLESTYLE)
+	    continue;
+
+	/* two blank lines between tabulated plots by prepending an empty line here */
+	print_line("");
+	snprintf(line, size, "# Curve %d of %d, %d points",
 		curve, plot_num, current_plot->p_count);
+	print_line(line);
 
 	if ((current_plot->title) && (*current_plot->title)) {
 	    char *title = expand_newline(current_plot->title);
-    	    fprintf(outfile, "\n# Curve title: \"%s\"", title);
+	    snprintf(line, size, "# Curve title: \"%s\"", title);
+	    print_line(line);
 	    free(title);
 	}
 
-	fprintf(outfile, "\n# x y");
+	len = snprintf(line, size, "# x y");
 	switch (current_plot->plot_style) {
 	case BOXES:
 	case XERRORBARS:
-	    fputs(" xlow xhigh", outfile);
+	    len = strappend(&line, &size, len, " xlow xhigh");
 	    break;
 	case BOXERROR:
 	case YERRORBARS:
-	    fputs(" ylow yhigh", outfile);
+	    len = strappend(&line, &size, len, " ylow yhigh");
 	    break;
 	case BOXXYERROR:
 	case XYERRORBARS:
-	    fputs(" xlow xhigh ylow yhigh", outfile);
+	    len = strappend(&line, &size, len, " xlow xhigh ylow yhigh");
 	    break;
 	case FILLEDCURVES:
-	    fputs("1 y2", outfile);
+	    len = strappend(&line, &size, len, "1 y2");
 	    break;
 	case FINANCEBARS:
-	    fputs(" open ylow yhigh yclose", outfile);
+	    len = strappend(&line, &size, len, " open ylow yhigh yclose");
 	    break;
 	case CANDLESTICKS:
-	    fputs(" open ylow yhigh yclose width", outfile);
+	    len = strappend(&line, &size, len, " open ylow yhigh yclose width");
 	    break;
 	case LABELPOINTS:
-	    fputs(" label",outfile);
+	    len = strappend(&line, &size, len, " label");
 	    break;
 	case VECTOR:
-	    fputs(" delta_x delta_y",outfile);
+	    len = strappend(&line, &size, len, " delta_x delta_y");
 	    break;
 	case LINES:
 	case POINTSTYLE:
@@ -147,11 +189,11 @@ print_table(struct curve_points *current_plot, int plot_num)
 	case HISTEPS:
 	    break;
 	case IMAGE:
-	    fputs("  pixel", outfile);
+	    len = strappend(&line, &size, len, "  pixel");
 	    break;
 	case RGBIMAGE:
 	case RGBA_IMAGE:
-	    fputs("  red green blue alpha", outfile);
+	    len = strappend(&line, &size, len, "  red green blue alpha");
 	    break;
 
 	default:
@@ -163,23 +205,33 @@ print_table(struct curve_points *current_plot, int plot_num)
 	}
 
 	if (current_plot->varcolor)
-	    fputs("  color", outfile);
+	    len = strappend(&line, &size, len, "  color");
 
-	fputs(" type\n", outfile);
+	strappend(&line, &size, len, " type");
+	print_line(line);
 
 	if (current_plot->plot_style == LABELPOINTS) {
 	    struct text_label *this_label;
-	    for (this_label = current_plot->labels->next; this_label != NULL;
+	    for (this_label = current_plot->labels->next;
+		 this_label != NULL;
 		 this_label = this_label->next) {
-		 char *label = expand_newline(this_label->text);
-		 OUTPUT_NUMBER(this_label->place.x, current_plot->x_axis);
-		 OUTPUT_NUMBER(this_label->place.y, current_plot->y_axis);
-		fprintf(outfile, " \"%s\"\n", label);
+		char *label = expand_newline(this_label->text);
+		line[0] = NUL;
+		len = 0;
+		OUTPUT_NUMBER(this_label->place.x, current_plot->x_axis);
+		OUTPUT_NUMBER(this_label->place.y, current_plot->y_axis);
+		len = strappend(&line, &size, len, "\"");
+		len = strappend(&line, &size, len, label);
+		len = strappend(&line, &size, len, "\"");
+		print_line(line);
 		free(label);
 	    }
 
 	} else {
 	    int plotstyle = current_plot->plot_style;
+	    int type;
+	    TBOOLEAN replace_undefined_with_blank = imploded(current_plot);
+
 	    if (plotstyle == HISTOGRAMS && current_plot->histogram->type == HT_ERRORBARS)
 		plotstyle = YERRORBARS;
 
@@ -188,11 +240,13 @@ print_table(struct curve_points *current_plot, int plot_num)
 
 		/* Reproduce blank lines read from original input file, if any */
 		if (!memcmp(point, &blank_data_line, sizeof(struct coordinate))) {
-		    fprintf(outfile, "\n");
+		    print_line("");
 		    continue;
 		}
 
 		/* FIXME HBB 20020405: had better use the real x/x2 axes of this plot */
+		line[0] = NUL;
+		len = 0;
 		OUTPUT_NUMBER(point->x, current_plot->x_axis);
 		OUTPUT_NUMBER(point->y, current_plot->y_axis);
 
@@ -215,14 +269,15 @@ print_table(struct curve_points *current_plot, int plot_num)
 			OUTPUT_NUMBER(point->yhigh, current_plot->y_axis);
 			break;
 		    case IMAGE:
-			fprintf(outfile,"%g ",point->z);
+			snprintf(buffer, BUFFERSIZE, "%g ", point->z);
+			len = strappend(&line, &size, len, buffer);
 			break;
 		    case RGBIMAGE:
 		    case RGBA_IMAGE:
-			fprintf(outfile,"%4d ",(int)point->CRD_R);
-			fprintf(outfile,"%4d ",(int)point->CRD_G);
-			fprintf(outfile,"%4d ",(int)point->CRD_B);
-			fprintf(outfile,"%4d ",(int)point->CRD_A);
+			snprintf(buffer, BUFFERSIZE, "%4d %4d %4d %4d ", 
+			        (int)point->CRD_R, (int)point->CRD_G,
+			        (int)point->CRD_B, (int)point->CRD_A);
+			len = strappend(&line, &size, len, buffer);
 			break;
 		    case FILLEDCURVES:
 			OUTPUT_NUMBER(point->yhigh, current_plot->y_axis);
@@ -260,7 +315,8 @@ print_table(struct curve_points *current_plot, int plot_num)
 		    double colorval = current_plot->varcolor[i];
 		    if ((current_plot->lp_properties.pm3d_color.value < 0.0)
 		    &&  (current_plot->lp_properties.pm3d_color.type == TC_RGB)) {
-			fprintf(outfile, "0x%6x", (unsigned int)(colorval));
+			snprintf(buffer, BUFFERSIZE, "0x%06x", (unsigned int)(colorval));
+			len = strappend(&line, &size, len, buffer);
 		    } else if (current_plot->lp_properties.pm3d_color.type == TC_Z) {
 			OUTPUT_NUMBER(colorval, COLOR_AXIS);
 		    } else if (current_plot->lp_properties.l_type == LT_COLORFROMCOLUMN) {
@@ -268,18 +324,29 @@ print_table(struct curve_points *current_plot, int plot_num)
 		    }
 		}
 
-		fprintf(outfile, " %c\n",
-		    current_plot->points[i].type == INRANGE
-		    ? 'i' : current_plot->points[i].type == OUTRANGE
-		    ? 'o' : 'u');
+		type = current_plot->points[i].type;
+
+		snprintf(buffer, BUFFERSIZE, " %c",
+			type == INRANGE ? 'i' : type == OUTRANGE ? 'o' : 'u');
+		strappend(&line, &size, len, buffer);
+
+		/* cp_implode() inserts dummy undefined point between curves */
+		/* but datafiles use a blank line for this purpose */
+		if (type == UNDEFINED && replace_undefined_with_blank)
+		    print_line("");
+		else 
+		    print_line(line);
+
 	    } /* for(point i) */
 	}
 
-	putc('\n', outfile);
+	print_line("");
     } /* for(curve) */
 
-    fflush(outfile);
+    if (outfile)
+	fflush(outfile);
     free(buffer);
+    free(line);
 }
 
 
@@ -288,19 +355,27 @@ print_3dtable(int pcount)
 {
     struct surface_points *this_plot;
     int i, surface;
-    struct coordinate GPHUGE *point;
-    struct coordinate GPHUGE *tail;
-    char *buffer = gp_alloc(BUFFERSIZE, "print_3dtable output buffer");
+    struct coordinate *point;
+    struct coordinate *tail;
+    char *buffer = (char *) gp_alloc(BUFFERSIZE, "print_3dtable: output buffer");
+    size_t size = 2*BUFFERSIZE;
+    char *line = (char *) gp_alloc(size, "print_3dtable: line buffer");
+    size_t len = 0;
+
     outfile = (table_outfile) ? table_outfile : gpoutfile;
 
     for (surface = 0, this_plot = first_3dplot;
 	 surface < pcount;
 	 this_plot = this_plot->next_sp, surface++) {
-	fprintf(outfile, "\n# Surface %d of %d surfaces\n", surface, pcount);
+	print_line("");
+	snprintf(line, size, "# Surface %d of %d surfaces", surface, pcount);
+	print_line(line);
 
 	if ((this_plot->title) && (*this_plot->title)) {
 	    char *title = expand_newline(this_plot->title);
-    	    fprintf(outfile, "\n# Curve title: \"%s\"", title);
+	    print_line("");
+	    snprintf(line, size, "# Curve title: \"%s\"", title);
+	    print_line(line);
 	    free(title);
 	}
 
@@ -308,13 +383,19 @@ print_3dtable(int pcount)
 	case LABELPOINTS:
 	    {
 	    struct text_label *this_label;
-	    for (this_label = this_plot->labels->next; this_label != NULL;
+	    for (this_label = this_plot->labels->next;
+		 this_label != NULL;
 		 this_label = this_label->next) {
-		 char *label = expand_newline(this_label->text);
-		 OUTPUT_NUMBER(this_label->place.x, FIRST_X_AXIS);
-		 OUTPUT_NUMBER(this_label->place.y, FIRST_Y_AXIS);
-		 OUTPUT_NUMBER(this_label->place.z, FIRST_Z_AXIS);
-		fprintf(outfile, " \"%s\"\n", label);
+		char *label = expand_newline(this_label->text);
+		line[0] = NUL;
+		len = 0;
+		OUTPUT_NUMBER(this_label->place.x, FIRST_X_AXIS);
+		OUTPUT_NUMBER(this_label->place.y, FIRST_Y_AXIS);
+		OUTPUT_NUMBER(this_label->place.z, FIRST_Z_AXIS);
+		len = strappend(&line, &size, len, "\"");
+		len = strappend(&line, &size, len, label);
+		len = strappend(&line, &size, len, "\"");
+		print_line(line);
 		free(label);
 	    }
 	    }
@@ -340,18 +421,26 @@ print_3dtable(int pcount)
 		 icrvs && curve < this_plot->num_iso_read;
 		 icrvs = icrvs->next, curve++) {
 
-		fprintf(outfile, "\n# IsoCurve %d, %d points\n# x y z",
+		print_line("");
+		snprintf(line, size, "# IsoCurve %d, %d points",
 			curve, icrvs->p_count);
+		print_line(line);
+		len = sprintf(line, "# x y z");
 		if (this_plot->plot_style == VECTOR) {
 		    tail = icrvs->next->points;
-		    fprintf(outfile, " delta_x delta_y delta_z");
-		} else tail = NULL;  /* Just to shut up a compiler warning */
+		    len = strappend(&line, &size, len, " delta_x delta_y delta_z");
+		} else {
+		    tail = NULL;  /* Just to shut up a compiler warning */
+		}
 
-		fprintf(outfile, " type\n");
+		strappend(&line, &size, len, " type");
+		print_line(line);
 
 		for (i = 0, point = icrvs->points;
 		     i < icrvs->p_count;
 		     i++, point++) {
+		    line[0] = NUL;
+		    len = 0;
 		    OUTPUT_NUMBER(point->x, FIRST_X_AXIS);
 		    OUTPUT_NUMBER(point->y, FIRST_Y_AXIS);
 		    OUTPUT_NUMBER(point->z, FIRST_Z_AXIS);
@@ -361,16 +450,19 @@ print_3dtable(int pcount)
 			OUTPUT_NUMBER((tail->z - point->z), FIRST_Z_AXIS);
 			tail++;
 		    } else if (this_plot->plot_style == IMAGE) {
-			fprintf(outfile,"%g ",point->CRD_COLOR);
+			snprintf(buffer, BUFFERSIZE, "%g ", point->CRD_COLOR);
+			len = strappend(&line, &size, len, buffer);
 		    }
-		    fprintf(outfile, "%c\n",
+		    snprintf(buffer, BUFFERSIZE, "%c",
 			    point->type == INRANGE
 			    ? 'i' : point->type == OUTRANGE
 			    ? 'o' : 'u');
-		} /* for(point) */
-	    } /* for(icrvs) */
-	    putc('\n', outfile);
-	} /* if(draw_surface) */
+		    strappend(&line, &size, len, buffer);
+		    print_line(line);
+		} /* for (point) */
+	    } /* for (icrvs) */
+	    print_line("");
+	} /* if (draw_surface) */
 
 	if (draw_contour) {
 	    int number = 0;
@@ -378,38 +470,45 @@ print_3dtable(int pcount)
 
 	    while (c) {
 		int count = c->num_pts;
-		struct coordinate GPHUGE *point = c->coords;
+		struct coordinate *point = c->coords;
 
 		if (c->isNewLevel)
 		    /* don't display count - contour split across chunks */
 		    /* put # in case user wants to use it for a plot */
 		    /* double blank line to allow plot ... index ... */
-		    fprintf(outfile, "\n# Contour %d, label: %s\n",
+		    print_line("");
+		    snprintf(line, size, "# Contour %d, label: %s",
 			    number++, c->label);
+		    print_line(line);
 
 		for (; --count >= 0; ++point) {
+		    line[0] = NUL;
+		    len = 0;
 		    OUTPUT_NUMBER(point->x, FIRST_X_AXIS);
 		    OUTPUT_NUMBER(point->y, FIRST_Y_AXIS);
 		    OUTPUT_NUMBER(point->z, FIRST_Z_AXIS);
-		    putc('\n', outfile);
+		    print_line(line);
 		}
 
 		/* blank line between segments of same contour */
-		putc('\n', outfile);
+		print_line("");
 		c = c->next;
 
 	    } /* while (contour) */
 	} /* if (draw_contour) */
     } /* for(surface) */
-    fflush(outfile);
 
+    if (outfile)
+	fflush(outfile);
     free(buffer);
+    free(line);
 }
 
+
 static char *
 expand_newline(const char *in)
 {
-    char *tmpstr = gp_alloc(2*strlen(in),"enl");
+    char *tmpstr = (char *) gp_alloc(2 * strlen(in), "enl");
     const char *s = in;
     char *t = tmpstr;
     do {
@@ -421,3 +520,67 @@ expand_newline(const char *in)
     } while (*s++);
     return tmpstr;
 }
+
+static TBOOLEAN
+imploded(curve_points *this_plot)
+{
+    switch (this_plot->plot_smooth) {
+	/* These smooth styles called cp_implode() */
+	case SMOOTH_UNIQUE:
+	case SMOOTH_FREQUENCY:
+	case SMOOTH_FREQUENCY_NORMALISED:
+	case SMOOTH_CUMULATIVE:
+	case SMOOTH_CUMULATIVE_NORMALISED:
+	case SMOOTH_CSPLINES:
+	case SMOOTH_ACSPLINES:
+	case SMOOTH_SBEZIER:
+	case SMOOTH_MONOTONE_CSPLINE:
+	    return TRUE;
+	/* These ones did not */
+	case SMOOTH_NONE:
+	case SMOOTH_BEZIER:
+	case SMOOTH_KDENSITY:
+	default:
+	    break;
+    }
+    return FALSE;
+}
+
+/*
+ * Called from plot2d.c (get_data) for "plot with table"
+ */
+TBOOLEAN
+tabulate_one_line(double v[MAXDATACOLS], struct value str[MAXDATACOLS], int ncols)
+{
+    int col;
+    FILE *outfile = (table_outfile) ? table_outfile : gpoutfile;
+
+    if (table_var == NULL) {
+	char sep = (table_sep && *table_sep) ? *table_sep : '\t';
+	for (col = 0; col < ncols; col++) {
+	    if (str[col].type == STRING)
+		fprintf(outfile, " %s%c", str[col].v.string_val, sep);
+	    else
+		fprintf(outfile, " %g%c", v[col], sep);
+	}
+	fprintf(outfile, "\n");
+    } else {
+	char buf[64]; /* buffer large enough to hold %g + 2 extra chars */
+	char sep = (table_sep && *table_sep) ? *table_sep : '\t';
+	size_t size = sizeof(buf);
+	char *line = (char *) gp_alloc(size, "");
+	size_t len = 0;
+
+	line[0] = NUL;
+	for (col = 0; col < ncols; col++) {
+	    if (str[col].type == STRING)
+		snprintf(buf, sizeof(buf), " %s%c", str[col].v.string_val, sep);
+	    else
+		snprintf(buf, sizeof(buf), " %g%c", v[col], sep);
+	    len = strappend(&line, &size, len, buf);
+	}
+	append_to_datablock(&table_var->udv_value, line);
+    }
+ 
+    return TRUE;
+}
